from _typeshed import Incomplete

MAXPNAMELEN: int
MAXERRORLENGTH: int
MAX_JOYSTICKOEMVXDNAME: int
MM_MICROSOFT: int
MM_MIDI_MAPPER: int
MM_WAVE_MAPPER: int
MM_SNDBLST_MIDIOUT: int
MM_SNDBLST_MIDIIN: int
MM_SNDBLST_SYNTH: int
MM_SNDBLST_WAVEOUT: int
MM_SNDBLST_WAVEIN: int
MM_ADLIB: int
MM_MPU401_MIDIOUT: int
MM_MPU401_MIDIIN: int
MM_PC_JOYSTICK: int
TIME_MS: int
TIME_SAMPLES: int
TIME_BYTES: int
TIME_SMPTE: int
TIME_MIDI: int
TIME_TICKS: int
MM_JOY1MOVE: int
MM_JOY2MOVE: int
MM_JOY1ZMOVE: int
MM_JOY2ZMOVE: int
MM_JOY1BUTTONDOWN: int
MM_JOY2BUTTONDOWN: int
MM_JOY1BUTTONUP: int
MM_JOY2BUTTONUP: int
MM_MCINOTIFY: int
MM_WOM_OPEN: int
MM_WOM_CLOSE: int
MM_WOM_DONE: int
MM_WIM_OPEN: int
MM_WIM_CLOSE: int
MM_WIM_DATA: int
MM_MIM_OPEN: int
MM_MIM_CLOSE: int
MM_MIM_DATA: int
MM_MIM_LONGDATA: int
MM_MIM_ERROR: int
MM_MIM_LONGERROR: int
MM_MOM_OPEN: int
MM_MOM_CLOSE: int
MM_MOM_DONE: int
MM_STREAM_OPEN: int
MM_STREAM_CLOSE: int
MM_STREAM_DONE: int
MM_STREAM_ERROR: int
MM_MOM_POSITIONCB: int
MM_MIM_MOREDATA: int
MM_MIXM_LINE_CHANGE: int
MM_MIXM_CONTROL_CHANGE: int
MMSYSERR_BASE: int
WAVERR_BASE: int
MIDIERR_BASE: int
TIMERR_BASE: int
JOYERR_BASE: int
MCIERR_BASE: int
MIXERR_BASE: int
MCI_STRING_OFFSET: int
MCI_VD_OFFSET: int
MCI_CD_OFFSET: int
MCI_WAVE_OFFSET: int
MCI_SEQ_OFFSET: int
MMSYSERR_NOERROR: int
MMSYSERR_ERROR: Incomplete
MMSYSERR_BADDEVICEID: Incomplete
MMSYSERR_NOTENABLED: Incomplete
MMSYSERR_ALLOCATED: Incomplete
MMSYSERR_INVALHANDLE: Incomplete
MMSYSERR_NODRIVER: Incomplete
MMSYSERR_NOMEM: Incomplete
MMSYSERR_NOTSUPPORTED: Incomplete
MMSYSERR_BADERRNUM: Incomplete
MMSYSERR_INVALFLAG: Incomplete
MMSYSERR_INVALPARAM: Incomplete
MMSYSERR_HANDLEBUSY: Incomplete
MMSYSERR_INVALIDALIAS: Incomplete
MMSYSERR_BADDB: Incomplete
MMSYSERR_KEYNOTFOUND: Incomplete
MMSYSERR_READERROR: Incomplete
MMSYSERR_WRITEERROR: Incomplete
MMSYSERR_DELETEERROR: Incomplete
MMSYSERR_VALNOTFOUND: Incomplete
MMSYSERR_NODRIVERCB: Incomplete
MMSYSERR_LASTERROR: Incomplete
DRV_LOAD: int
DRV_ENABLE: int
DRV_OPEN: int
DRV_CLOSE: int
DRV_DISABLE: int
DRV_FREE: int
DRV_CONFIGURE: int
DRV_QUERYCONFIGURE: int
DRV_INSTALL: int
DRV_REMOVE: int
DRV_EXITSESSION: int
DRV_POWER: int
DRV_RESERVED: int
DRV_USER: int
DRVCNF_CANCEL: int
DRVCNF_OK: int
DRVCNF_RESTART: int
DRV_CANCEL: int
DRV_OK: int
DRV_RESTART: int
DRV_MCI_FIRST: int
DRV_MCI_LAST: Incomplete
CALLBACK_TYPEMASK: int
CALLBACK_NULL: int
CALLBACK_WINDOW: int
CALLBACK_TASK: int
CALLBACK_FUNCTION: int
CALLBACK_THREAD: int
CALLBACK_EVENT: int
SND_SYNC: int
SND_ASYNC: int
SND_NODEFAULT: int
SND_MEMORY: int
SND_LOOP: int
SND_NOSTOP: int
SND_NOWAIT: int
SND_ALIAS: int
SND_ALIAS_ID: int
SND_FILENAME: int
SND_RESOURCE: int
SND_PURGE: int
SND_APPLICATION: int
SND_ALIAS_START: int
WAVERR_BADFORMAT: Incomplete
WAVERR_STILLPLAYING: Incomplete
WAVERR_UNPREPARED: Incomplete
WAVERR_SYNC: Incomplete
WAVERR_LASTERROR: Incomplete
WOM_OPEN: int
WOM_CLOSE: int
WOM_DONE: int
WIM_OPEN: int
WIM_CLOSE: int
WIM_DATA: int
WAVE_MAPPER: int
WAVE_FORMAT_QUERY: int
WAVE_ALLOWSYNC: int
WAVE_MAPPED: int
WAVE_FORMAT_DIRECT: int
WAVE_FORMAT_DIRECT_QUERY: Incomplete
WHDR_DONE: int
WHDR_PREPARED: int
WHDR_BEGINLOOP: int
WHDR_ENDLOOP: int
WHDR_INQUEUE: int
WAVECAPS_PITCH: int
WAVECAPS_PLAYBACKRATE: int
WAVECAPS_VOLUME: int
WAVECAPS_LRVOLUME: int
WAVECAPS_SYNC: int
WAVECAPS_SAMPLEACCURATE: int
WAVECAPS_DIRECTSOUND: int
WAVE_INVALIDFORMAT: int
WAVE_FORMAT_1M08: int
WAVE_FORMAT_1S08: int
WAVE_FORMAT_1M16: int
WAVE_FORMAT_1S16: int
WAVE_FORMAT_2M08: int
WAVE_FORMAT_2S08: int
WAVE_FORMAT_2M16: int
WAVE_FORMAT_2S16: int
WAVE_FORMAT_4M08: int
WAVE_FORMAT_4S08: int
WAVE_FORMAT_4M16: int
WAVE_FORMAT_4S16: int
WAVE_FORMAT_PCM: int
WAVE_FORMAT_IEEE_FLOAT: int
MIDIERR_UNPREPARED: Incomplete
MIDIERR_STILLPLAYING: Incomplete
MIDIERR_NOMAP: Incomplete
MIDIERR_NOTREADY: Incomplete
MIDIERR_NODEVICE: Incomplete
MIDIERR_INVALIDSETUP: Incomplete
MIDIERR_BADOPENMODE: Incomplete
MIDIERR_DONT_CONTINUE: Incomplete
MIDIERR_LASTERROR: Incomplete
MIDIPATCHSIZE: int
MIM_OPEN: int
MIM_CLOSE: int
MIM_DATA: int
MIM_LONGDATA: int
MIM_ERROR: int
MIM_LONGERROR: int
MOM_OPEN: int
MOM_CLOSE: int
MOM_DONE: int
MIM_MOREDATA: int
MOM_POSITIONCB: int
MIDI_IO_STATUS: int
MIDI_CACHE_ALL: int
MIDI_CACHE_BESTFIT: int
MIDI_CACHE_QUERY: int
MIDI_UNCACHE: int
MOD_MIDIPORT: int
MOD_SYNTH: int
MOD_SQSYNTH: int
MOD_FMSYNTH: int
MOD_MAPPER: int
MIDICAPS_VOLUME: int
MIDICAPS_LRVOLUME: int
MIDICAPS_CACHE: int
MIDICAPS_STREAM: int
MHDR_DONE: int
MHDR_PREPARED: int
MHDR_INQUEUE: int
MHDR_ISSTRM: int
MEVT_F_SHORT: int
MEVT_F_LONG: int
MEVT_F_CALLBACK: int

def MEVT_EVENTTYPE(x): ...
def MEVT_EVENTPARM(x): ...

MIDISTRM_ERROR: int
MIDIPROP_SET: int
MIDIPROP_GET: int
MIDIPROP_TIMEDIV: int
MIDIPROP_TEMPO: int
AUXCAPS_CDAUDIO: int
AUXCAPS_AUXIN: int
AUXCAPS_VOLUME: int
AUXCAPS_LRVOLUME: int
MIXER_SHORT_NAME_CHARS: int
MIXER_LONG_NAME_CHARS: int
MIXERR_INVALLINE: Incomplete
MIXERR_INVALCONTROL: Incomplete
MIXERR_INVALVALUE: Incomplete
MIXERR_LASTERROR: Incomplete
MIXER_OBJECTF_HANDLE: int
MIXER_OBJECTF_MIXER: int
MIXER_OBJECTF_HMIXER: Incomplete
MIXER_OBJECTF_WAVEOUT: int
MIXER_OBJECTF_HWAVEOUT: Incomplete
MIXER_OBJECTF_WAVEIN: int
MIXER_OBJECTF_HWAVEIN: Incomplete
MIXER_OBJECTF_MIDIOUT: int
MIXER_OBJECTF_HMIDIOUT: Incomplete
MIXER_OBJECTF_MIDIIN: int
MIXER_OBJECTF_HMIDIIN: Incomplete
MIXER_OBJECTF_AUX: int
MIXERLINE_LINEF_ACTIVE: int
MIXERLINE_LINEF_DISCONNECTED: int
MIXERLINE_LINEF_SOURCE: int
MIXERLINE_COMPONENTTYPE_DST_FIRST: int
MIXERLINE_COMPONENTTYPE_DST_UNDEFINED: Incomplete
MIXERLINE_COMPONENTTYPE_DST_DIGITAL: Incomplete
MIXERLINE_COMPONENTTYPE_DST_LINE: Incomplete
MIXERLINE_COMPONENTTYPE_DST_MONITOR: Incomplete
MIXERLINE_COMPONENTTYPE_DST_SPEAKERS: Incomplete
MIXERLINE_COMPONENTTYPE_DST_HEADPHONES: Incomplete
MIXERLINE_COMPONENTTYPE_DST_TELEPHONE: Incomplete
MIXERLINE_COMPONENTTYPE_DST_WAVEIN: Incomplete
MIXERLINE_COMPONENTTYPE_DST_VOICEIN: Incomplete
MIXERLINE_COMPONENTTYPE_DST_LAST: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_FIRST: int
MIXERLINE_COMPONENTTYPE_SRC_UNDEFINED: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_DIGITAL: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_LINE: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_SYNTHESIZER: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_COMPACTDISC: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_TELEPHONE: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_PCSPEAKER: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_AUXILIARY: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_ANALOG: Incomplete
MIXERLINE_COMPONENTTYPE_SRC_LAST: Incomplete
MIXERLINE_TARGETTYPE_UNDEFINED: int
MIXERLINE_TARGETTYPE_WAVEOUT: int
MIXERLINE_TARGETTYPE_WAVEIN: int
MIXERLINE_TARGETTYPE_MIDIOUT: int
MIXERLINE_TARGETTYPE_MIDIIN: int
MIXERLINE_TARGETTYPE_AUX: int
MIXER_GETLINEINFOF_DESTINATION: int
MIXER_GETLINEINFOF_SOURCE: int
MIXER_GETLINEINFOF_LINEID: int
MIXER_GETLINEINFOF_COMPONENTTYPE: int
MIXER_GETLINEINFOF_TARGETTYPE: int
MIXER_GETLINEINFOF_QUERYMASK: int
MIXERCONTROL_CONTROLF_UNIFORM: int
MIXERCONTROL_CONTROLF_MULTIPLE: int
MIXERCONTROL_CONTROLF_DISABLED: int
MIXERCONTROL_CT_CLASS_MASK: int
MIXERCONTROL_CT_CLASS_CUSTOM: int
MIXERCONTROL_CT_CLASS_METER: int
MIXERCONTROL_CT_CLASS_SWITCH: int
MIXERCONTROL_CT_CLASS_NUMBER: int
MIXERCONTROL_CT_CLASS_SLIDER: int
MIXERCONTROL_CT_CLASS_FADER: int
MIXERCONTROL_CT_CLASS_TIME: int
MIXERCONTROL_CT_CLASS_LIST: int
MIXERCONTROL_CT_SUBCLASS_MASK: int
MIXERCONTROL_CT_SC_SWITCH_BOOLEAN: int
MIXERCONTROL_CT_SC_SWITCH_BUTTON: int
MIXERCONTROL_CT_SC_METER_POLLED: int
MIXERCONTROL_CT_SC_TIME_MICROSECS: int
MIXERCONTROL_CT_SC_TIME_MILLISECS: int
MIXERCONTROL_CT_SC_LIST_SINGLE: int
MIXERCONTROL_CT_SC_LIST_MULTIPLE: int
MIXERCONTROL_CT_UNITS_MASK: int
MIXERCONTROL_CT_UNITS_CUSTOM: int
MIXERCONTROL_CT_UNITS_BOOLEAN: int
MIXERCONTROL_CT_UNITS_SIGNED: int
MIXERCONTROL_CT_UNITS_UNSIGNED: int
MIXERCONTROL_CT_UNITS_DECIBELS: int
MIXERCONTROL_CT_UNITS_PERCENT: int
MIXERCONTROL_CONTROLTYPE_CUSTOM: Incomplete
MIXERCONTROL_CONTROLTYPE_BOOLEANMETER: Incomplete
MIXERCONTROL_CONTROLTYPE_SIGNEDMETER: Incomplete
MIXERCONTROL_CONTROLTYPE_PEAKMETER: Incomplete
MIXERCONTROL_CONTROLTYPE_UNSIGNEDMETER: Incomplete
MIXERCONTROL_CONTROLTYPE_BOOLEAN: Incomplete
MIXERCONTROL_CONTROLTYPE_ONOFF: Incomplete
MIXERCONTROL_CONTROLTYPE_MUTE: Incomplete
MIXERCONTROL_CONTROLTYPE_MONO: Incomplete
MIXERCONTROL_CONTROLTYPE_LOUDNESS: Incomplete
MIXERCONTROL_CONTROLTYPE_STEREOENH: Incomplete
MIXERCONTROL_CONTROLTYPE_BUTTON: Incomplete
MIXERCONTROL_CONTROLTYPE_DECIBELS: Incomplete
MIXERCONTROL_CONTROLTYPE_SIGNED: Incomplete
MIXERCONTROL_CONTROLTYPE_UNSIGNED: Incomplete
MIXERCONTROL_CONTROLTYPE_PERCENT: Incomplete
MIXERCONTROL_CONTROLTYPE_SLIDER: Incomplete
MIXERCONTROL_CONTROLTYPE_PAN: Incomplete
MIXERCONTROL_CONTROLTYPE_QSOUNDPAN: Incomplete
MIXERCONTROL_CONTROLTYPE_FADER: Incomplete
MIXERCONTROL_CONTROLTYPE_VOLUME: Incomplete
MIXERCONTROL_CONTROLTYPE_BASS: Incomplete
MIXERCONTROL_CONTROLTYPE_TREBLE: Incomplete
MIXERCONTROL_CONTROLTYPE_EQUALIZER: Incomplete
MIXERCONTROL_CONTROLTYPE_SINGLESELECT: Incomplete
MIXERCONTROL_CONTROLTYPE_MUX: Incomplete
MIXERCONTROL_CONTROLTYPE_MULTIPLESELECT: Incomplete
MIXERCONTROL_CONTROLTYPE_MIXER: Incomplete
MIXERCONTROL_CONTROLTYPE_MICROTIME: Incomplete
MIXERCONTROL_CONTROLTYPE_MILLITIME: Incomplete
MIXER_GETLINECONTROLSF_ALL: int
MIXER_GETLINECONTROLSF_ONEBYID: int
MIXER_GETLINECONTROLSF_ONEBYTYPE: int
MIXER_GETLINECONTROLSF_QUERYMASK: int
MIXER_GETCONTROLDETAILSF_VALUE: int
MIXER_GETCONTROLDETAILSF_LISTTEXT: int
MIXER_GETCONTROLDETAILSF_QUERYMASK: int
MIXER_SETCONTROLDETAILSF_VALUE: int
MIXER_SETCONTROLDETAILSF_CUSTOM: int
MIXER_SETCONTROLDETAILSF_QUERYMASK: int
TIMERR_NOERROR: int
TIMERR_NOCANDO: Incomplete
TIMERR_STRUCT: Incomplete
TIME_ONESHOT: int
TIME_PERIODIC: int
TIME_CALLBACK_FUNCTION: int
TIME_CALLBACK_EVENT_SET: int
TIME_CALLBACK_EVENT_PULSE: int
JOYERR_NOERROR: int
JOYERR_PARMS: Incomplete
JOYERR_NOCANDO: Incomplete
JOYERR_UNPLUGGED: Incomplete
JOY_BUTTON1: int
JOY_BUTTON2: int
JOY_BUTTON3: int
JOY_BUTTON4: int
JOY_BUTTON1CHG: int
JOY_BUTTON2CHG: int
JOY_BUTTON3CHG: int
JOY_BUTTON4CHG: int
JOY_BUTTON5: int
JOY_BUTTON6: int
JOY_BUTTON7: int
JOY_BUTTON8: int
JOY_BUTTON9: int
JOY_BUTTON10: int
JOY_BUTTON11: int
JOY_BUTTON12: int
JOY_BUTTON13: int
JOY_BUTTON14: int
JOY_BUTTON15: int
JOY_BUTTON16: int
JOY_BUTTON17: int
JOY_BUTTON18: int
JOY_BUTTON19: int
JOY_BUTTON20: int
JOY_BUTTON21: int
JOY_BUTTON22: int
JOY_BUTTON23: int
JOY_BUTTON24: int
JOY_BUTTON25: int
JOY_BUTTON26: int
JOY_BUTTON27: int
JOY_BUTTON28: int
JOY_BUTTON29: int
JOY_BUTTON30: int
JOY_BUTTON31: int
JOY_BUTTON32: int
JOY_POVFORWARD: int
JOY_POVRIGHT: int
JOY_POVBACKWARD: int
JOY_POVLEFT: int
JOY_RETURNX: int
JOY_RETURNY: int
JOY_RETURNZ: int
JOY_RETURNR: int
JOY_RETURNU: int
JOY_RETURNV: int
JOY_RETURNPOV: int
JOY_RETURNBUTTONS: int
JOY_RETURNRAWDATA: int
JOY_RETURNPOVCTS: int
JOY_RETURNCENTERED: int
JOY_USEDEADZONE: int
JOY_RETURNALL: Incomplete
JOY_CAL_READALWAYS: int
JOY_CAL_READXYONLY: int
JOY_CAL_READ3: int
JOY_CAL_READ4: int
JOY_CAL_READXONLY: int
JOY_CAL_READYONLY: int
JOY_CAL_READ5: int
JOY_CAL_READ6: int
JOY_CAL_READZONLY: int
JOY_CAL_READRONLY: int
JOY_CAL_READUONLY: int
JOY_CAL_READVONLY: int
JOYSTICKID1: int
JOYSTICKID2: int
JOYCAPS_HASZ: int
JOYCAPS_HASR: int
JOYCAPS_HASU: int
JOYCAPS_HASV: int
JOYCAPS_HASPOV: int
JOYCAPS_POV4DIR: int
JOYCAPS_POVCTS: int
MMIOERR_BASE: int
MMIOERR_FILENOTFOUND: Incomplete
MMIOERR_OUTOFMEMORY: Incomplete
MMIOERR_CANNOTOPEN: Incomplete
MMIOERR_CANNOTCLOSE: Incomplete
MMIOERR_CANNOTREAD: Incomplete
MMIOERR_CANNOTWRITE: Incomplete
MMIOERR_CANNOTSEEK: Incomplete
MMIOERR_CANNOTEXPAND: Incomplete
MMIOERR_CHUNKNOTFOUND: Incomplete
MMIOERR_UNBUFFERED: Incomplete
MMIOERR_PATHNOTFOUND: Incomplete
MMIOERR_ACCESSDENIED: Incomplete
MMIOERR_SHARINGVIOLATION: Incomplete
MMIOERR_NETWORKERROR: Incomplete
MMIOERR_TOOMANYOPENFILES: Incomplete
MMIOERR_INVALIDFILE: Incomplete
CFSEPCHAR: Incomplete
MMIO_RWMODE: int
MMIO_SHAREMODE: int
MMIO_CREATE: int
MMIO_PARSE: int
MMIO_DELETE: int
MMIO_EXIST: int
MMIO_ALLOCBUF: int
MMIO_GETTEMP: int
MMIO_DIRTY: int
MMIO_READ: int
MMIO_WRITE: int
MMIO_READWRITE: int
MMIO_COMPAT: int
MMIO_EXCLUSIVE: int
MMIO_DENYWRITE: int
MMIO_DENYREAD: int
MMIO_DENYNONE: int
MMIO_FHOPEN: int
MMIO_EMPTYBUF: int
MMIO_TOUPPER: int
MMIO_INSTALLPROC: int
MMIO_GLOBALPROC: int
MMIO_REMOVEPROC: int
MMIO_UNICODEPROC: int
MMIO_FINDPROC: int
MMIO_FINDCHUNK: int
MMIO_FINDRIFF: int
MMIO_FINDLIST: int
MMIO_CREATERIFF: int
MMIO_CREATELIST: int
MMIOM_READ: int
MMIOM_WRITE: int
MMIOM_SEEK: int
MMIOM_OPEN: int
MMIOM_CLOSE: int
MMIOM_WRITEFLUSH: int
MMIOM_RENAME: int
MMIOM_USER: int
SEEK_SET: int
SEEK_CUR: int
SEEK_END: int
MMIO_DEFAULTBUFFER: int
MCIERR_INVALID_DEVICE_ID: Incomplete
MCIERR_UNRECOGNIZED_KEYWORD: Incomplete
MCIERR_UNRECOGNIZED_COMMAND: Incomplete
MCIERR_HARDWARE: Incomplete
MCIERR_INVALID_DEVICE_NAME: Incomplete
MCIERR_OUT_OF_MEMORY: Incomplete
MCIERR_DEVICE_OPEN: Incomplete
MCIERR_CANNOT_LOAD_DRIVER: Incomplete
MCIERR_MISSING_COMMAND_STRING: Incomplete
MCIERR_PARAM_OVERFLOW: Incomplete
MCIERR_MISSING_STRING_ARGUMENT: Incomplete
MCIERR_BAD_INTEGER: Incomplete
MCIERR_PARSER_INTERNAL: Incomplete
MCIERR_DRIVER_INTERNAL: Incomplete
MCIERR_MISSING_PARAMETER: Incomplete
MCIERR_UNSUPPORTED_FUNCTION: Incomplete
MCIERR_FILE_NOT_FOUND: Incomplete
MCIERR_DEVICE_NOT_READY: Incomplete
MCIERR_INTERNAL: Incomplete
MCIERR_DRIVER: Incomplete
MCIERR_CANNOT_USE_ALL: Incomplete
MCIERR_MULTIPLE: Incomplete
MCIERR_EXTENSION_NOT_FOUND: Incomplete
MCIERR_OUTOFRANGE: Incomplete
MCIERR_FLAGS_NOT_COMPATIBLE: Incomplete
MCIERR_FILE_NOT_SAVED: Incomplete
MCIERR_DEVICE_TYPE_REQUIRED: Incomplete
MCIERR_DEVICE_LOCKED: Incomplete
MCIERR_DUPLICATE_ALIAS: Incomplete
MCIERR_BAD_CONSTANT: Incomplete
MCIERR_MUST_USE_SHAREABLE: Incomplete
MCIERR_MISSING_DEVICE_NAME: Incomplete
MCIERR_BAD_TIME_FORMAT: Incomplete
MCIERR_NO_CLOSING_QUOTE: Incomplete
MCIERR_DUPLICATE_FLAGS: Incomplete
MCIERR_INVALID_FILE: Incomplete
MCIERR_NULL_PARAMETER_BLOCK: Incomplete
MCIERR_UNNAMED_RESOURCE: Incomplete
MCIERR_NEW_REQUIRES_ALIAS: Incomplete
MCIERR_NOTIFY_ON_AUTO_OPEN: Incomplete
MCIERR_NO_ELEMENT_ALLOWED: Incomplete
MCIERR_NONAPPLICABLE_FUNCTION: Incomplete
MCIERR_ILLEGAL_FOR_AUTO_OPEN: Incomplete
MCIERR_FILENAME_REQUIRED: Incomplete
MCIERR_EXTRA_CHARACTERS: Incomplete
MCIERR_DEVICE_NOT_INSTALLED: Incomplete
MCIERR_GET_CD: Incomplete
MCIERR_SET_CD: Incomplete
MCIERR_SET_DRIVE: Incomplete
MCIERR_DEVICE_LENGTH: Incomplete
MCIERR_DEVICE_ORD_LENGTH: Incomplete
MCIERR_NO_INTEGER: Incomplete
MCIERR_WAVE_OUTPUTSINUSE: Incomplete
MCIERR_WAVE_SETOUTPUTINUSE: Incomplete
MCIERR_WAVE_INPUTSINUSE: Incomplete
MCIERR_WAVE_SETINPUTINUSE: Incomplete
MCIERR_WAVE_OUTPUTUNSPECIFIED: Incomplete
MCIERR_WAVE_INPUTUNSPECIFIED: Incomplete
MCIERR_WAVE_OUTPUTSUNSUITABLE: Incomplete
MCIERR_WAVE_SETOUTPUTUNSUITABLE: Incomplete
MCIERR_WAVE_INPUTSUNSUITABLE: Incomplete
MCIERR_WAVE_SETINPUTUNSUITABLE: Incomplete
MCIERR_SEQ_DIV_INCOMPATIBLE: Incomplete
MCIERR_SEQ_PORT_INUSE: Incomplete
MCIERR_SEQ_PORT_NONEXISTENT: Incomplete
MCIERR_SEQ_PORT_MAPNODEVICE: Incomplete
MCIERR_SEQ_PORT_MISCERROR: Incomplete
MCIERR_SEQ_TIMER: Incomplete
MCIERR_SEQ_PORTUNSPECIFIED: Incomplete
MCIERR_SEQ_NOMIDIPRESENT: Incomplete
MCIERR_NO_WINDOW: Incomplete
MCIERR_CREATEWINDOW: Incomplete
MCIERR_FILE_READ: Incomplete
MCIERR_FILE_WRITE: Incomplete
MCIERR_NO_IDENTITY: Incomplete
MCIERR_CUSTOM_DRIVER_BASE: Incomplete
MCI_FIRST: int
MCI_OPEN: int
MCI_CLOSE: int
MCI_ESCAPE: int
MCI_PLAY: int
MCI_SEEK: int
MCI_STOP: int
MCI_PAUSE: int
MCI_INFO: int
MCI_GETDEVCAPS: int
MCI_SPIN: int
MCI_SET: int
MCI_STEP: int
MCI_RECORD: int
MCI_SYSINFO: int
MCI_BREAK: int
MCI_SAVE: int
MCI_STATUS: int
MCI_CUE: int
MCI_REALIZE: int
MCI_WINDOW: int
MCI_PUT: int
MCI_WHERE: int
MCI_FREEZE: int
MCI_UNFREEZE: int
MCI_LOAD: int
MCI_CUT: int
MCI_COPY: int
MCI_PASTE: int
MCI_UPDATE: int
MCI_RESUME: int
MCI_DELETE: int
MCI_USER_MESSAGES: Incomplete
MCI_LAST: int
MCI_DEVTYPE_VCR: int
MCI_DEVTYPE_VIDEODISC: int
MCI_DEVTYPE_OVERLAY: int
MCI_DEVTYPE_CD_AUDIO: int
MCI_DEVTYPE_DAT: int
MCI_DEVTYPE_SCANNER: int
MCI_DEVTYPE_ANIMATION: int
MCI_DEVTYPE_DIGITAL_VIDEO: int
MCI_DEVTYPE_OTHER: int
MCI_DEVTYPE_WAVEFORM_AUDIO: int
MCI_DEVTYPE_SEQUENCER: int
MCI_DEVTYPE_FIRST: int
MCI_DEVTYPE_LAST: int
MCI_DEVTYPE_FIRST_USER: int
MCI_MODE_NOT_READY: Incomplete
MCI_MODE_STOP: Incomplete
MCI_MODE_PLAY: Incomplete
MCI_MODE_RECORD: Incomplete
MCI_MODE_SEEK: Incomplete
MCI_MODE_PAUSE: Incomplete
MCI_MODE_OPEN: Incomplete
MCI_FORMAT_MILLISECONDS: int
MCI_FORMAT_HMS: int
MCI_FORMAT_MSF: int
MCI_FORMAT_FRAMES: int
MCI_FORMAT_SMPTE_24: int
MCI_FORMAT_SMPTE_25: int
MCI_FORMAT_SMPTE_30: int
MCI_FORMAT_SMPTE_30DROP: int
MCI_FORMAT_BYTES: int
MCI_FORMAT_SAMPLES: int
MCI_FORMAT_TMSF: int

def MCI_MSF_MINUTE(msf): ...
def MCI_MSF_SECOND(msf): ...
def MCI_MSF_FRAME(msf): ...
def MCI_TMSF_TRACK(tmsf): ...
def MCI_TMSF_MINUTE(tmsf): ...
def MCI_TMSF_SECOND(tmsf): ...
def MCI_TMSF_FRAME(tmsf): ...
def MCI_HMS_HOUR(hms): ...
def MCI_HMS_MINUTE(hms): ...
def MCI_HMS_SECOND(hms): ...

MCI_NOTIFY_SUCCESSFUL: int
MCI_NOTIFY_SUPERSEDED: int
MCI_NOTIFY_ABORTED: int
MCI_NOTIFY_FAILURE: int
MCI_NOTIFY: int
MCI_WAIT: int
MCI_FROM: int
MCI_TO: int
MCI_TRACK: int
MCI_OPEN_SHAREABLE: int
MCI_OPEN_ELEMENT: int
MCI_OPEN_ALIAS: int
MCI_OPEN_ELEMENT_ID: int
MCI_OPEN_TYPE_ID: int
MCI_OPEN_TYPE: int
MCI_SEEK_TO_START: int
MCI_SEEK_TO_END: int
MCI_STATUS_ITEM: int
MCI_STATUS_START: int
MCI_STATUS_LENGTH: int
MCI_STATUS_POSITION: int
MCI_STATUS_NUMBER_OF_TRACKS: int
MCI_STATUS_MODE: int
MCI_STATUS_MEDIA_PRESENT: int
MCI_STATUS_TIME_FORMAT: int
MCI_STATUS_READY: int
MCI_STATUS_CURRENT_TRACK: int
MCI_INFO_PRODUCT: int
MCI_INFO_FILE: int
MCI_INFO_MEDIA_UPC: int
MCI_INFO_MEDIA_IDENTITY: int
MCI_INFO_NAME: int
MCI_INFO_COPYRIGHT: int
MCI_GETDEVCAPS_ITEM: int
MCI_GETDEVCAPS_CAN_RECORD: int
MCI_GETDEVCAPS_HAS_AUDIO: int
MCI_GETDEVCAPS_HAS_VIDEO: int
MCI_GETDEVCAPS_DEVICE_TYPE: int
MCI_GETDEVCAPS_USES_FILES: int
MCI_GETDEVCAPS_COMPOUND_DEVICE: int
MCI_GETDEVCAPS_CAN_EJECT: int
MCI_GETDEVCAPS_CAN_PLAY: int
MCI_GETDEVCAPS_CAN_SAVE: int
MCI_SYSINFO_QUANTITY: int
MCI_SYSINFO_OPEN: int
MCI_SYSINFO_NAME: int
MCI_SYSINFO_INSTALLNAME: int
MCI_SET_DOOR_OPEN: int
MCI_SET_DOOR_CLOSED: int
MCI_SET_TIME_FORMAT: int
MCI_SET_AUDIO: int
MCI_SET_VIDEO: int
MCI_SET_ON: int
MCI_SET_OFF: int
MCI_SET_AUDIO_ALL: int
MCI_SET_AUDIO_LEFT: int
MCI_SET_AUDIO_RIGHT: int
MCI_BREAK_KEY: int
MCI_BREAK_HWND: int
MCI_BREAK_OFF: int
MCI_RECORD_INSERT: int
MCI_RECORD_OVERWRITE: int
MCI_SAVE_FILE: int
MCI_LOAD_FILE: int
MCI_VD_MODE_PARK: Incomplete
MCI_VD_MEDIA_CLV: Incomplete
MCI_VD_MEDIA_CAV: Incomplete
MCI_VD_MEDIA_OTHER: Incomplete
MCI_VD_FORMAT_TRACK: int
MCI_VD_PLAY_REVERSE: int
MCI_VD_PLAY_FAST: int
MCI_VD_PLAY_SPEED: int
MCI_VD_PLAY_SCAN: int
MCI_VD_PLAY_SLOW: int
MCI_VD_SEEK_REVERSE: int
MCI_VD_STATUS_SPEED: int
MCI_VD_STATUS_FORWARD: int
MCI_VD_STATUS_MEDIA_TYPE: int
MCI_VD_STATUS_SIDE: int
MCI_VD_STATUS_DISC_SIZE: int
MCI_VD_GETDEVCAPS_CLV: int
MCI_VD_GETDEVCAPS_CAV: int
MCI_VD_SPIN_UP: int
MCI_VD_SPIN_DOWN: int
MCI_VD_GETDEVCAPS_CAN_REVERSE: int
MCI_VD_GETDEVCAPS_FAST_RATE: int
MCI_VD_GETDEVCAPS_SLOW_RATE: int
MCI_VD_GETDEVCAPS_NORMAL_RATE: int
MCI_VD_STEP_FRAMES: int
MCI_VD_STEP_REVERSE: int
MCI_VD_ESCAPE_STRING: int
MCI_CDA_STATUS_TYPE_TRACK: int
MCI_CDA_TRACK_AUDIO: Incomplete
MCI_CDA_TRACK_OTHER: Incomplete
MCI_WAVE_PCM: Incomplete
MCI_WAVE_MAPPER: Incomplete
MCI_WAVE_OPEN_BUFFER: int
MCI_WAVE_SET_FORMATTAG: int
MCI_WAVE_SET_CHANNELS: int
MCI_WAVE_SET_SAMPLESPERSEC: int
MCI_WAVE_SET_AVGBYTESPERSEC: int
MCI_WAVE_SET_BLOCKALIGN: int
MCI_WAVE_SET_BITSPERSAMPLE: int
MCI_WAVE_INPUT: int
MCI_WAVE_OUTPUT: int
MCI_WAVE_STATUS_FORMATTAG: int
MCI_WAVE_STATUS_CHANNELS: int
MCI_WAVE_STATUS_SAMPLESPERSEC: int
MCI_WAVE_STATUS_AVGBYTESPERSEC: int
MCI_WAVE_STATUS_BLOCKALIGN: int
MCI_WAVE_STATUS_BITSPERSAMPLE: int
MCI_WAVE_STATUS_LEVEL: int
MCI_WAVE_SET_ANYINPUT: int
MCI_WAVE_SET_ANYOUTPUT: int
MCI_WAVE_GETDEVCAPS_INPUTS: int
MCI_WAVE_GETDEVCAPS_OUTPUTS: int
MCI_SEQ_DIV_PPQN: Incomplete
MCI_SEQ_DIV_SMPTE_24: Incomplete
MCI_SEQ_DIV_SMPTE_25: Incomplete
MCI_SEQ_DIV_SMPTE_30DROP: Incomplete
MCI_SEQ_DIV_SMPTE_30: Incomplete
MCI_SEQ_FORMAT_SONGPTR: int
MCI_SEQ_FILE: int
MCI_SEQ_MIDI: int
MCI_SEQ_SMPTE: int
MCI_SEQ_NONE: int
MCI_SEQ_MAPPER: int
MCI_SEQ_STATUS_TEMPO: int
MCI_SEQ_STATUS_PORT: int
MCI_SEQ_STATUS_SLAVE: int
MCI_SEQ_STATUS_MASTER: int
MCI_SEQ_STATUS_OFFSET: int
MCI_SEQ_STATUS_DIVTYPE: int
MCI_SEQ_STATUS_NAME: int
MCI_SEQ_STATUS_COPYRIGHT: int
MCI_SEQ_SET_TEMPO: int
MCI_SEQ_SET_PORT: int
MCI_SEQ_SET_SLAVE: int
MCI_SEQ_SET_MASTER: int
MCI_SEQ_SET_OFFSET: int
MCI_ANIM_OPEN_WS: int
MCI_ANIM_OPEN_PARENT: int
MCI_ANIM_OPEN_NOSTATIC: int
MCI_ANIM_PLAY_SPEED: int
MCI_ANIM_PLAY_REVERSE: int
MCI_ANIM_PLAY_FAST: int
MCI_ANIM_PLAY_SLOW: int
MCI_ANIM_PLAY_SCAN: int
MCI_ANIM_STEP_REVERSE: int
MCI_ANIM_STEP_FRAMES: int
MCI_ANIM_STATUS_SPEED: int
MCI_ANIM_STATUS_FORWARD: int
MCI_ANIM_STATUS_HWND: int
MCI_ANIM_STATUS_HPAL: int
MCI_ANIM_STATUS_STRETCH: int
MCI_ANIM_INFO_TEXT: int
MCI_ANIM_GETDEVCAPS_CAN_REVERSE: int
MCI_ANIM_GETDEVCAPS_FAST_RATE: int
MCI_ANIM_GETDEVCAPS_SLOW_RATE: int
MCI_ANIM_GETDEVCAPS_NORMAL_RATE: int
MCI_ANIM_GETDEVCAPS_PALETTES: int
MCI_ANIM_GETDEVCAPS_CAN_STRETCH: int
MCI_ANIM_GETDEVCAPS_MAX_WINDOWS: int
MCI_ANIM_REALIZE_NORM: int
MCI_ANIM_REALIZE_BKGD: int
MCI_ANIM_WINDOW_HWND: int
MCI_ANIM_WINDOW_STATE: int
MCI_ANIM_WINDOW_TEXT: int
MCI_ANIM_WINDOW_ENABLE_STRETCH: int
MCI_ANIM_WINDOW_DISABLE_STRETCH: int
MCI_ANIM_WINDOW_DEFAULT: int
MCI_ANIM_RECT: int
MCI_ANIM_PUT_SOURCE: int
MCI_ANIM_PUT_DESTINATION: int
MCI_ANIM_WHERE_SOURCE: int
MCI_ANIM_WHERE_DESTINATION: int
MCI_ANIM_UPDATE_HDC: int
MCI_OVLY_OPEN_WS: int
MCI_OVLY_OPEN_PARENT: int
MCI_OVLY_STATUS_HWND: int
MCI_OVLY_STATUS_STRETCH: int
MCI_OVLY_INFO_TEXT: int
MCI_OVLY_GETDEVCAPS_CAN_STRETCH: int
MCI_OVLY_GETDEVCAPS_CAN_FREEZE: int
MCI_OVLY_GETDEVCAPS_MAX_WINDOWS: int
MCI_OVLY_WINDOW_HWND: int
MCI_OVLY_WINDOW_STATE: int
MCI_OVLY_WINDOW_TEXT: int
MCI_OVLY_WINDOW_ENABLE_STRETCH: int
MCI_OVLY_WINDOW_DISABLE_STRETCH: int
MCI_OVLY_WINDOW_DEFAULT: int
MCI_OVLY_RECT: int
MCI_OVLY_PUT_SOURCE: int
MCI_OVLY_PUT_DESTINATION: int
MCI_OVLY_PUT_FRAME: int
MCI_OVLY_PUT_VIDEO: int
MCI_OVLY_WHERE_SOURCE: int
MCI_OVLY_WHERE_DESTINATION: int
MCI_OVLY_WHERE_FRAME: int
MCI_OVLY_WHERE_VIDEO: int
SELECTDIB: int

def DIBINDEX(n): ...



from collections.abc import Iterable
from typing import Any

from .resource import Model

class Swarm(Model):
    id_attribute: str
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def version(self) -> str | None: ...
    def get_unlock_key(self) -> dict[str, Any]: ...
    def init(
        self,
        advertise_addr: str | None = None,
        listen_addr: str = "0.0.0.0:2377",
        force_new_cluster: bool = False,
        default_addr_pool: Iterable[str] | None = None,
        subnet_size: int | None = None,
        data_path_addr: str | None = None,
        data_path_port: int | None = None,
        **kwargs,
    ) -> str: ...
    def join(self, *args, **kwargs) -> bool: ...
    def leave(self, *args, **kwargs) -> bool: ...
    def reload(self) -> None: ...
    def unlock(self, key: str) -> bool: ...
    def update(
        self,
        rotate_worker_token: bool = False,
        rotate_manager_token: bool = False,
        rotate_manager_unlock_key: bool = False,
        **kwargs,
    ) -> bool: ...



from typing import Final

XK_kra: Final = 0x3A2
XK_kappa: Final = 0x3A2
XK_Rcedilla: Final = 0x3A3
XK_Itilde: Final = 0x3A5
XK_Lcedilla: Final = 0x3A6
XK_Emacron: Final = 0x3AA
XK_Gcedilla: Final = 0x3AB
XK_Tslash: Final = 0x3AC
XK_rcedilla: Final = 0x3B3
XK_itilde: Final = 0x3B5
XK_lcedilla: Final = 0x3B6
XK_emacron: Final = 0x3BA
XK_gcedilla: Final = 0x3BB
XK_tslash: Final = 0x3BC
XK_ENG: Final = 0x3BD
XK_eng: Final = 0x3BF
XK_Amacron: Final = 0x3C0
XK_Iogonek: Final = 0x3C7
XK_Eabovedot: Final = 0x3CC
XK_Imacron: Final = 0x3CF
XK_Ncedilla: Final = 0x3D1
XK_Omacron: Final = 0x3D2
XK_Kcedilla: Final = 0x3D3
XK_Uogonek: Final = 0x3D9
XK_Utilde: Final = 0x3DD
XK_Umacron: Final = 0x3DE
XK_amacron: Final = 0x3E0
XK_iogonek: Final = 0x3E7
XK_eabovedot: Final = 0x3EC
XK_imacron: Final = 0x3EF
XK_ncedilla: Final = 0x3F1
XK_omacron: Final = 0x3F2
XK_kcedilla: Final = 0x3F3
XK_uogonek: Final = 0x3F9
XK_utilde: Final = 0x3FD
XK_umacron: Final = 0x3FE



import ast

def get_sim101(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...
def get_sim109(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...
def get_sim220(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...
def get_sim221(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...
def get_sim222(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...
def get_sim223(node: ast.BoolOp) -> list[tuple[int, int, str]]: ...



from ratelimit.decorators import RateLimitDecorator, sleep_and_retry
from ratelimit.exception import RateLimitException

limits = RateLimitDecorator
rate_limited = RateLimitDecorator

__all__ = ["RateLimitException", "limits", "rate_limited", "sleep_and_retry"]



debug: bool

class Error(Exception):
    msg: str
    def __init__(self, msg: str) -> None: ...

class ValidationError(Error):
    msg: str
    file: str
    def __init__(self, msg: str, file: str) -> None: ...

class ParsingError(Error):
    msg: str
    file: str
    def __init__(self, msg: str, file: str) -> None: ...

class NoKeyError(Error):
    key: str
    group: str
    file: str
    def __init__(self, key: str, group: str, file: str) -> None: ...

class DuplicateKeyError(Error):
    key: str
    group: str
    file: str
    def __init__(self, key: str, group: str, file: str) -> None: ...

class NoGroupError(Error):
    group: str
    file: str
    def __init__(self, group: str, file: str) -> None: ...

class DuplicateGroupError(Error):
    group: str
    file: str
    def __init__(self, group: str, file: str) -> None: ...

class NoThemeError(Error):
    theme: str
    def __init__(self, theme: str) -> None: ...



from collections.abc import Collection
from typing_extensions import Self

from .base import BaseMultipartGeometry
from .multilinestring import MultiLineString
from .polygon import Polygon, _PolygonHolesLike, _PolygonShellLike

__all__ = ["MultiPolygon"]

class MultiPolygon(BaseMultipartGeometry[Polygon]):
    __slots__: list[str] = []
    def __new__(
        self,
        polygons: (
            BaseMultipartGeometry
            | Collection[Polygon | tuple[_PolygonShellLike] | tuple[_PolygonShellLike, _PolygonHolesLike] | None]
            | None
        ) = None,
    ) -> Self: ...
    def svg(self, scale_factor: float = 1.0, fill_color: str | None = None, opacity: float | None = None) -> str: ...  # type: ignore[override]
    # more precise base overrides
    @property
    def boundary(self) -> MultiLineString: ...



############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import cattrs
from typing import Any, Callable, List, Optional, Union

from lsprotocol.types import (
    ALL_TYPES_MAP,
    METHOD_TO_TYPES,
    TEXT_DOCUMENT_DID_SAVE,
    TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL,
    TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL_DELTA,
    TEXT_DOCUMENT_SEMANTIC_TOKENS_RANGE,
    WORKSPACE_DID_CREATE_FILES,
    WORKSPACE_DID_DELETE_FILES,
    WORKSPACE_DID_RENAME_FILES,
    WORKSPACE_WILL_CREATE_FILES,
    WORKSPACE_WILL_DELETE_FILES,
    WORKSPACE_WILL_RENAME_FILES,
    FileOperationRegistrationOptions,
    SaveOptions,
    SemanticTokensLegend,
    SemanticTokensRegistrationOptions,
    ShowDocumentResult,
)

from pygls.exceptions import MethodTypeNotRegisteredError

ConfigCallbackType = Callable[[List[Any]], None]
ShowDocumentCallbackType = Callable[[ShowDocumentResult], None]

METHOD_TO_OPTIONS = {
    TEXT_DOCUMENT_DID_SAVE: SaveOptions,
    TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL: Union[
        SemanticTokensLegend, SemanticTokensRegistrationOptions
    ],
    TEXT_DOCUMENT_SEMANTIC_TOKENS_FULL_DELTA: Union[
        SemanticTokensLegend, SemanticTokensRegistrationOptions
    ],
    TEXT_DOCUMENT_SEMANTIC_TOKENS_RANGE: Union[
        SemanticTokensLegend, SemanticTokensRegistrationOptions
    ],
    WORKSPACE_DID_CREATE_FILES: FileOperationRegistrationOptions,
    WORKSPACE_DID_DELETE_FILES: FileOperationRegistrationOptions,
    WORKSPACE_DID_RENAME_FILES: FileOperationRegistrationOptions,
    WORKSPACE_WILL_CREATE_FILES: FileOperationRegistrationOptions,
    WORKSPACE_WILL_DELETE_FILES: FileOperationRegistrationOptions,
    WORKSPACE_WILL_RENAME_FILES: FileOperationRegistrationOptions,
}


def get_method_registration_options_type(
    method_name, lsp_methods_map=METHOD_TO_TYPES
) -> Optional[Any]:
    """The type corresponding with a method's options when dynamically registering
    capability for it."""

    try:
        return lsp_methods_map[method_name][3]
    except KeyError:
        raise MethodTypeNotRegisteredError(method_name)


def get_method_options_type(
    method_name, lsp_options_map=METHOD_TO_OPTIONS, lsp_methods_map=METHOD_TO_TYPES
) -> Optional[Any]:
    """Return the type corresponding with a method's ``ServerCapabilities`` fields.

    In the majority of cases this simply means returning the ``<MethodName>Options``
    type, which we can easily derive from the method's
    ``<MethodName>RegistrationOptions`` type.

    However, where the options are more involved (such as semantic tokens) and
    ``pygls`` does some extra work to help derive the options for the user the type
    has to be provided via the ``lsp_options_map``

    Arguments:
        method_name:
            The lsp method name to retrieve the options for

        lsp_options_map:
            The map used to override the default options type finding behavior

        lsp_methods_map:
            The standard map used to look up the various method types.
    """

    options_type = lsp_options_map.get(method_name, None)
    if options_type is not None:
        return options_type

    registration_type = get_method_registration_options_type(
        method_name, lsp_methods_map
    )
    if registration_type is None:
        return None

    type_name = registration_type.__name__.replace("Registration", "")
    options_type = ALL_TYPES_MAP.get(type_name, None)

    if options_type is None:
        raise MethodTypeNotRegisteredError(method_name)

    return options_type


def get_method_params_type(method_name, lsp_methods_map=METHOD_TO_TYPES):
    try:
        return lsp_methods_map[method_name][2]
    except KeyError:
        raise MethodTypeNotRegisteredError(method_name)


def get_method_return_type(method_name, lsp_methods_map=METHOD_TO_TYPES):
    try:
        return lsp_methods_map[method_name][1]
    except KeyError:
        raise MethodTypeNotRegisteredError(method_name)


def is_instance(cv: cattrs.Converter, o, t):
    try:
        cv.unstructure(o, t)
        return True
    except TypeError:
        return False



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Sequence
from typing import SupportsIndex

if sys.platform != "win32":
    if sys.version_info >= (3, 14):
        def fork_exec(
            args: Sequence[StrOrBytesPath] | None,
            executable_list: Sequence[bytes],
            close_fds: bool,
            pass_fds: tuple[int, ...],
            cwd: str,
            env: Sequence[bytes] | None,
            p2cread: int,
            p2cwrite: int,
            c2pread: int,
            c2pwrite: int,
            errread: int,
            errwrite: int,
            errpipe_read: int,
            errpipe_write: int,
            restore_signals: int,
            call_setsid: int,
            pgid_to_set: int,
            gid: SupportsIndex | None,
            extra_groups: list[int] | None,
            uid: SupportsIndex | None,
            child_umask: int,
            preexec_fn: Callable[[], None],
            /,
        ) -> int: ...
    else:
        def fork_exec(
            args: Sequence[StrOrBytesPath] | None,
            executable_list: Sequence[bytes],
            close_fds: bool,
            pass_fds: tuple[int, ...],
            cwd: str,
            env: Sequence[bytes] | None,
            p2cread: int,
            p2cwrite: int,
            c2pread: int,
            c2pwrite: int,
            errread: int,
            errwrite: int,
            errpipe_read: int,
            errpipe_write: int,
            restore_signals: bool,
            call_setsid: bool,
            pgid_to_set: int,
            gid: SupportsIndex | None,
            extra_groups: list[int] | None,
            uid: SupportsIndex | None,
            child_umask: int,
            preexec_fn: Callable[[], None],
            allow_vfork: bool,
            /,
        ) -> int: ...



from collections.abc import Mapping
from logging import Logger

from oauthlib.common import Request, _HTTPMethod
from oauthlib.oauth2.rfc6749.endpoints.base import BaseEndpoint
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator as OAuth2RequestValidator
from oauthlib.oauth2.rfc6749.tokens import BearerToken

log: Logger

class UserInfoEndpoint(BaseEndpoint):
    bearer: BearerToken
    request_validator: OAuth2RequestValidator
    def __init__(self, request_validator: OAuth2RequestValidator) -> None: ...
    def create_userinfo_response(
        self,
        uri: str,
        http_method: _HTTPMethod = "GET",
        body: str | dict[str, str] | list[tuple[str, str]] | None = None,
        headers: Mapping[str, str] | None = None,
    ) -> tuple[dict[str, str], str, int]: ...
    def validate_userinfo_request(self, request: Request) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Add a dummy package name. Having no package, like
core/lib/core/error_codes.proto, or having tensorflow.error, like
tsl/protobuf/error_codes.proto, results in name collision errors in generated
code for some users that use JS through J2CL.
"""

import google.protobuf.descriptor
from tensorflow.tsl.protobuf.error_codes_pb2 import (
    ABORTED as ABORTED,
    ALREADY_EXISTS as ALREADY_EXISTS,
    CANCELLED as CANCELLED,
    DATA_LOSS as DATA_LOSS,
    DEADLINE_EXCEEDED as DEADLINE_EXCEEDED,
    DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_ as DO_NOT_USE_RESERVED_FOR_FUTURE_EXPANSION_USE_DEFAULT_IN_SWITCH_INSTEAD_,
    FAILED_PRECONDITION as FAILED_PRECONDITION,
    INTERNAL as INTERNAL,
    INVALID_ARGUMENT as INVALID_ARGUMENT,
    NOT_FOUND as NOT_FOUND,
    OK as OK,
    OUT_OF_RANGE as OUT_OF_RANGE,
    PERMISSION_DENIED as PERMISSION_DENIED,
    RESOURCE_EXHAUSTED as RESOURCE_EXHAUSTED,
    UNAUTHENTICATED as UNAUTHENTICATED,
    UNAVAILABLE as UNAVAILABLE,
    UNIMPLEMENTED as UNIMPLEMENTED,
    UNKNOWN as UNKNOWN,
    Code as Code,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



import datetime
from _typeshed import Incomplete, SupportsItems
from collections.abc import Callable
from typing import Any, ClassVar, Final, Literal, NamedTuple, overload
from typing_extensions import Self

from .alarms import Alarms
from .caselessdict import CaselessDict
from .error import IncompleteComponent as IncompleteComponent
from .parser import Contentline, Contentlines
from .prop import TypesFactory, vRecur
from .timezone.tzp import TZP

__all__ = [
    "Alarm",
    "Calendar",
    "Component",
    "ComponentFactory",
    "Event",
    "FreeBusy",
    "INLINE",
    "Journal",
    "Timezone",
    "TimezoneDaylight",
    "TimezoneStandard",
    "Todo",
    "component_factory",
    "get_example",
    "IncompleteComponent",
]

def get_example(component_directory: str, example_name: str) -> bytes: ...

class ComponentFactory(CaselessDict[Incomplete]):
    def __init__(self, *args, **kwargs) -> None: ...

INLINE: CaselessDict[int]

class Component(CaselessDict[Incomplete]):
    name: ClassVar[str | None]
    required: ClassVar[tuple[str, ...]]
    singletons: ClassVar[tuple[str, ...]]
    multiple: ClassVar[tuple[str, ...]]
    exclusive: ClassVar[tuple[str, ...]]
    inclusive: ClassVar[tuple[tuple[str, ...], ...]]
    ignore_exceptions: ClassVar[bool]
    subcomponents: list[Incomplete]
    errors: list[str]

    def __init__(self, *args, **kwargs) -> None: ...
    def __bool__(self) -> bool: ...
    __nonzero__ = __bool__
    def is_empty(self) -> bool: ...
    @overload
    def add(self, name: str, value: Any, *, encode: Literal[False]) -> None: ...
    @overload
    def add(self, name: str, value: Any, parameters: None, encode: Literal[False]) -> None: ...
    @overload
    def add(
        self, name: str, value: Any, parameters: SupportsItems[str, str | None] | None = None, encode: Literal[True] = True
    ) -> None: ...
    def decoded(self, name, default=[]): ...
    def get_inline(self, name, decode: bool = True): ...
    def set_inline(self, name, values, encode: bool = True) -> None: ...
    def add_component(self, component: Component) -> None: ...
    def walk(self, name: str | None = None, select: Callable[[Component], bool] = ...) -> list[Component]: ...
    def property_items(self, recursive: bool = True, sorted: bool = True) -> list[tuple[str, object]]: ...
    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[False] = False) -> Component: ...  # or any of its subclasses
    @overload
    @classmethod
    def from_ical(cls, st: str, multiple: Literal[True]) -> list[Component]: ...  # or any of its subclasses
    def content_line(self, name: str, value, sorted: bool = True) -> Contentline: ...
    def content_lines(self, sorted: bool = True) -> Contentlines: ...
    def to_ical(self, sorted: bool = True) -> bytes: ...
    def __eq__(self, other: Component) -> bool: ...  # type: ignore[override]
    @property
    def DTSTAMP(self) -> datetime.datetime | None: ...
    @DTSTAMP.setter
    def DTSTAMP(self, value: datetime.datetime) -> None: ...
    @DTSTAMP.deleter
    def DTSTAMP(self) -> None: ...
    @property
    def LAST_MODIFIED(self) -> datetime.datetime | None: ...
    @LAST_MODIFIED.setter
    def LAST_MODIFIED(self, value: datetime.datetime) -> None: ...
    @LAST_MODIFIED.deleter
    def LAST_MODIFIED(self) -> None: ...
    def is_thunderbird(self) -> bool: ...

# type_def is a TypeForm
def create_single_property(
    prop: str, value_attr: str | None, value_type: tuple[type, ...], type_def: Any, doc: str, vProp: type[Incomplete] = ...
) -> property: ...

class Event(Component):
    name: ClassVar[Literal["VEVENT"]]
    @property
    def alarms(self) -> Alarms: ...
    @classmethod
    def example(cls, name: str = "rfc_9074_example_3") -> Event: ...
    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...
    @property
    def DTEND(self) -> datetime.date | datetime.datetime | None: ...
    @DTEND.setter
    def DTEND(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTEND.deleter
    def DTEND(self) -> None: ...
    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...
    @property
    def duration(self) -> datetime.timedelta: ...
    @property
    def start(self) -> datetime.date | datetime.datetime: ...
    @start.setter
    def start(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @property
    def end(self) -> datetime.date | datetime.datetime: ...
    @end.setter
    def end(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @property
    def X_MOZ_SNOOZE_TIME(self) -> datetime.datetime | None: ...
    @X_MOZ_SNOOZE_TIME.setter
    def X_MOZ_SNOOZE_TIME(self, value: datetime.datetime) -> None: ...
    @X_MOZ_SNOOZE_TIME.deleter
    def X_MOZ_SNOOZE_TIME(self) -> None: ...
    @property
    def X_MOZ_LASTACK(self) -> datetime.datetime | None: ...
    @X_MOZ_LASTACK.setter
    def X_MOZ_LASTACK(self, value: datetime.datetime) -> None: ...
    @X_MOZ_LASTACK.deleter
    def X_MOZ_LASTACK(self) -> None: ...
    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...
    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...
    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...
    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Todo(Component):
    name: ClassVar[Literal["VTODO"]]
    @property
    def DTSTART(self) -> datetime.datetime | datetime.date | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...
    @property
    def DUE(self) -> datetime.datetime | datetime.date | None: ...
    @DUE.setter
    def DUE(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @DUE.deleter
    def DUE(self) -> None: ...
    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...
    @property
    def start(self) -> datetime.datetime | datetime.date: ...
    @start.setter
    def start(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @property
    def end(self) -> datetime.datetime | datetime.date: ...
    @end.setter
    def end(self, value: datetime.datetime | datetime.date | None) -> None: ...
    @property
    def duration(self) -> datetime.timedelta: ...
    @property
    def X_MOZ_SNOOZE_TIME(self) -> datetime.datetime | None: ...
    @X_MOZ_SNOOZE_TIME.setter
    def X_MOZ_SNOOZE_TIME(self, value: datetime.datetime) -> None: ...
    @X_MOZ_SNOOZE_TIME.deleter
    def X_MOZ_SNOOZE_TIME(self) -> None: ...
    @property
    def X_MOZ_LASTACK(self) -> datetime.datetime | None: ...
    @X_MOZ_LASTACK.setter
    def X_MOZ_LASTACK(self, value: datetime.datetime) -> None: ...
    @X_MOZ_LASTACK.deleter
    def X_MOZ_LASTACK(self) -> None: ...
    @property
    def alarms(self) -> Alarms: ...
    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...
    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...
    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...
    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Journal(Component):
    name: ClassVar[Literal["VJOURNAL"]]
    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...
    @property
    def start(self) -> datetime.date | datetime.datetime: ...
    @start.setter
    def start(self, value: datetime.date | datetime.datetime | None) -> None: ...
    end = start
    @property
    def duration(self) -> datetime.timedelta: ...
    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...
    @property
    def sequence(self) -> int: ...
    @sequence.setter
    def sequence(self, value: int) -> None: ...
    @sequence.deleter
    def sequence(self) -> None: ...
    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...
    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class FreeBusy(Component):
    name: ClassVar[Literal["VFREEBUSY"]]
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Timezone(Component):
    subcomponents: list[TimezoneStandard | TimezoneDaylight]
    name: ClassVar[Literal["VTIMEZONE"]]
    @classmethod
    def example(cls, name: str = "pacific_fiji") -> Calendar: ...
    def to_tz(self, tzp: TZP = ..., lookup_tzid: bool = True) -> datetime.tzinfo: ...
    @property
    def tz_name(self) -> str: ...
    def get_transitions(self) -> tuple[list[datetime.datetime], list[tuple[datetime.timedelta, datetime.timedelta, str]]]: ...
    @classmethod
    def from_tzinfo(
        cls, timezone: datetime.tzinfo, tzid: str | None = None, first_date: datetime.date = ..., last_date: datetime.date = ...
    ) -> Self: ...
    @classmethod
    def from_tzid(cls, tzid: str, tzp: TZP = ..., first_date: datetime.date = ..., last_date: datetime.date = ...) -> Self: ...
    @property
    def standard(self) -> list[TimezoneStandard]: ...
    @property
    def daylight(self) -> list[TimezoneDaylight]: ...

class TimezoneStandard(Component):
    name: ClassVar[Literal["STANDARD"]]
    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...
    @property
    def TZOFFSETTO(self) -> datetime.timedelta | None: ...
    @TZOFFSETTO.setter
    def TZOFFSETTO(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETTO.deleter
    def TZOFFSETTO(self) -> None: ...
    @property
    def TZOFFSETFROM(self) -> datetime.timedelta | None: ...
    @TZOFFSETFROM.setter
    def TZOFFSETFROM(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETFROM.deleter
    def TZOFFSETFROM(self) -> None: ...
    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

class TimezoneDaylight(Component):
    name: ClassVar[Literal["DAYLIGHT"]]
    @property
    def DTSTART(self) -> datetime.date | datetime.datetime | None: ...
    @DTSTART.setter
    def DTSTART(self, value: datetime.date | datetime.datetime | None) -> None: ...
    @DTSTART.deleter
    def DTSTART(self) -> None: ...
    @property
    def TZOFFSETTO(self) -> datetime.timedelta | None: ...
    @TZOFFSETTO.setter
    def TZOFFSETTO(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETTO.deleter
    def TZOFFSETTO(self) -> None: ...
    @property
    def TZOFFSETFROM(self) -> datetime.timedelta | None: ...
    @TZOFFSETFROM.setter
    def TZOFFSETFROM(self, value: datetime.timedelta | None) -> None: ...
    @TZOFFSETFROM.deleter
    def TZOFFSETFROM(self) -> None: ...
    @property
    def rdates(
        self,
    ) -> list[tuple[datetime.date, None] | tuple[datetime.datetime, None] | tuple[datetime.datetime, datetime.datetime]]: ...
    @property
    def exdates(self) -> list[datetime.date | datetime.datetime]: ...
    @property
    def rrules(self) -> list[vRecur]: ...

class Alarm(Component):
    name: ClassVar[Literal["VALARM"]]
    @property
    def REPEAT(self) -> int: ...
    @REPEAT.setter
    def REPEAT(self, value: int) -> None: ...
    @REPEAT.deleter
    def REPEAT(self) -> None: ...
    @property
    def DURATION(self) -> datetime.timedelta | None: ...
    @DURATION.setter
    def DURATION(self, value: datetime.timedelta | None) -> None: ...
    @DURATION.deleter
    def DURATION(self) -> None: ...
    @property
    def ACKNOWLEDGED(self) -> datetime.datetime | None: ...
    @ACKNOWLEDGED.setter
    def ACKNOWLEDGED(self, value: datetime.datetime | None) -> None: ...
    @ACKNOWLEDGED.deleter
    def ACKNOWLEDGED(self) -> None: ...
    @property
    def TRIGGER(self) -> datetime.timedelta | datetime.datetime | None: ...
    @TRIGGER.setter
    def TRIGGER(self, value: datetime.timedelta | datetime.datetime | None) -> None: ...
    @TRIGGER.deleter
    def TRIGGER(self) -> None: ...
    @property
    def TRIGGER_RELATED(self) -> Literal["START", "END"]: ...
    @TRIGGER_RELATED.setter
    def TRIGGER_RELATED(self, value: Literal["START", "END"]) -> None: ...

    class Triggers(NamedTuple):
        start: tuple[datetime.timedelta, ...]
        end: tuple[datetime.timedelta, ...]
        absolute: tuple[datetime.datetime, ...]

    @property
    def triggers(self) -> Alarm.Triggers: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

class Calendar(Component):
    name: ClassVar[Literal["VCALENDAR"]]
    @classmethod
    def example(cls, name: str = "example") -> Calendar: ...
    @property
    def freebusy(self) -> list[FreeBusy]: ...
    @property
    def events(self) -> list[Event]: ...
    @property
    def todos(self) -> list[Todo]: ...
    def get_used_tzids(self) -> set[str]: ...
    def get_missing_tzids(self) -> set[str]: ...
    @property
    def timezones(self) -> list[Timezone]: ...
    def add_missing_timezones(self, first_date: datetime.date = ..., last_date: datetime.date = ...) -> None: ...
    @property
    def calendar_name(self) -> str | None: ...
    @calendar_name.setter
    def calendar_name(self, value: str) -> None: ...
    @calendar_name.deleter
    def calendar_name(self) -> None: ...
    @property
    def description(self) -> str | None: ...
    @description.setter
    def description(self, value: str) -> None: ...
    @description.deleter
    def description(self) -> None: ...
    @property
    def color(self) -> str: ...
    @color.setter
    def color(self, value: str) -> None: ...
    @color.deleter
    def color(self) -> None: ...
    @property
    def categories(self) -> list[str]: ...
    @categories.setter
    def categories(self, cats: list[str]) -> None: ...
    @categories.deleter
    def categories(self) -> None: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, value: str) -> None: ...
    @uid.deleter
    def uid(self) -> None: ...

types_factory: Final[TypesFactory]
component_factory: Final[ComponentFactory]



from ..timezone.tzp import TZP as TZP  # to prevent "tzp" from being defined here
from .tzid import tzid_from_dt as tzid_from_dt, tzid_from_tzinfo as tzid_from_tzinfo, tzids_from_tzinfo as tzids_from_tzinfo

__all__ = ["TZP", "tzp", "use_pytz", "use_zoneinfo", "tzid_from_tzinfo", "tzid_from_dt", "tzids_from_tzinfo"]

tzp: TZP

def use_pytz() -> None: ...
def use_zoneinfo() -> None: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.paypal_account import PayPalAccount
from braintree.successful_result import SuccessfulResult

class PayPalAccountGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def find(self, paypal_account_token: str) -> PayPalAccount | None: ...
    def delete(self, paypal_account_token: str) -> SuccessfulResult: ...
    def update(self, paypal_account_token: str, params=None) -> SuccessfulResult | ErrorResult | None: ...



import datetime
from typing import Literal, TypedDict, type_check_only
from typing_extensions import TypeAlias

from .date import DateDataParser, _DetectLanguagesFunction

__version__: str

_default_parser: DateDataParser

_Part: TypeAlias = Literal["day", "month", "year"]
_ParserKind: TypeAlias = Literal["timestamp", "relative-time", "custom-formats", "absolute-time", "no-spaces-time"]

@type_check_only
class _Settings(TypedDict, total=False):
    DATE_ORDER: str
    PREFER_LOCALE_DATE_ORDER: bool
    TIMEZONE: str
    TO_TIMEZONE: str
    RETURN_AS_TIMEZONE_AWARE: bool
    PREFER_DAY_OF_MONTH: Literal["current", "first", "last"]
    PREFER_DATES_FROM: Literal["current_period", "future", "past"]
    RELATIVE_BASE: datetime.datetime
    STRICT_PARSING: bool
    REQUIRE_PARTS: list[_Part]
    SKIP_TOKENS: list[str]
    NORMALIZE: bool
    RETURN_TIME_AS_PERIOD: bool
    PARSERS: list[_ParserKind]

def parse(
    date_string: str,
    date_formats: list[str] | tuple[str, ...] | set[str] | None = None,
    languages: list[str] | tuple[str, ...] | set[str] | None = None,
    locales: list[str] | tuple[str, ...] | set[str] | None = None,
    region: str | None = None,
    settings: _Settings | None = None,
    detect_languages_function: _DetectLanguagesFunction | None = None,
) -> datetime.datetime | None: ...



from typing import Any, Final

# String constant used to indicate all model fields should be included
ALL_FIELDS: Final[str] = "__all__"
# Collection of values considered empty by Django filters - tuple type allows various empty containers
EMPTY_VALUES: Final[Any] = ...



from _typeshed import Incomplete

class Formatter:
    pattern: Incomplete
    def __init__(self, pattern) -> None: ...
    def format(self, obj): ...
    def __call__(self, x): ...

class DecimalFormatter(Formatter):
    calcPlaces: Incomplete
    places: Incomplete
    dot: Incomplete
    comma: Incomplete
    prefix: Incomplete
    suffix: Incomplete
    def __init__(self, places: int = 2, decimalSep: str = ".", thousandSep=None, prefix=None, suffix=None) -> None: ...
    def format(self, num): ...

__all__ = ("Formatter", "DecimalFormatter")



from _typeshed import Incomplete

from antlr4.atn.ATNSimulator import ATNSimulator
from antlr4.error.ErrorListener import (
    ConsoleErrorListener as ConsoleErrorListener,
    ErrorListener,
    ProxyErrorListener as ProxyErrorListener,
)
from antlr4.error.Errors import RecognitionException
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token

class Recognizer:
    __slots__ = ("_listeners", "_interp", "_stateNumber")
    tokenTypeMapCache: dict[Incomplete, int]
    ruleIndexMapCache: dict[str, int]
    _listeners: list[ErrorListener]
    _interp: ATNSimulator | None
    _stateNumber: int
    def __init__(self) -> None: ...
    def extractVersion(self, version: str) -> tuple[str, str]: ...
    def checkVersion(self, toolVersion: str) -> None: ...
    def addErrorListener(self, listener: ErrorListener) -> None: ...
    def removeErrorListener(self, listener: ErrorListener) -> None: ...
    def removeErrorListeners(self) -> None: ...
    def getTokenTypeMap(self) -> dict[Incomplete, int]: ...
    def getRuleIndexMap(self) -> dict[str, int]: ...
    def getTokenType(self, tokenName: str) -> int: ...
    def getErrorHeader(self, e: RecognitionException) -> str: ...
    def getTokenErrorDisplay(self, t: Token | None) -> str: ...
    def getErrorListenerDispatch(self) -> ProxyErrorListener: ...
    def sempred(self, localctx: RuleContext, ruleIndex: int, actionIndex: int) -> bool: ...
    def precpred(self, localctx: RuleContext, precedence: int) -> bool: ...
    @property
    def state(self) -> int: ...
    @state.setter
    def state(self, atnState: int) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Sequence
from typing import Any, Generic, Literal, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias

import tensorflow as tf
from tensorflow import Tensor, Variable
from tensorflow._aliases import AnyArray, DataSequence, DTypeLike, Float, TensorCompatible, TensorLike
from tensorflow.keras.activations import _Activation
from tensorflow.keras.constraints import Constraint
from tensorflow.keras.initializers import _Initializer
from tensorflow.keras.regularizers import Regularizer, _Regularizer

_InputT_contra = TypeVar("_InputT_contra", contravariant=True)
_OutputT_co = TypeVar("_OutputT_co", covariant=True)

class InputSpec:
    dtype: str | None
    shape: tuple[int | None, ...]
    ndim: int | None
    max_ndim: int | None
    min_ndim: int | None
    axes: dict[int, int | None] | None
    def __init__(
        self,
        dtype: DTypeLike | None = None,
        shape: Iterable[int | None] | None = None,
        ndim: int | None = None,
        max_ndim: int | None = None,
        min_ndim: int | None = None,
        axes: dict[int, int | None] | None = None,
        allow_last_axis_squeeze: bool = False,
        name: str | None = None,
        optional: bool = False,
    ) -> None: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> type[Self]: ...

# Most layers have input and output type of just Tensor and when we support default type variables,
# maybe worth trying.
class Layer(tf.Module, Generic[_InputT_contra, _OutputT_co]):
    # The most general type is ContainerGeneric[InputSpec] as it really
    # depends on _InputT_contra. For most Layers it is just InputSpec
    # though. Maybe describable with HKT?
    input_spec: InputSpec | Any

    @property
    def trainable(self) -> bool: ...
    @trainable.setter
    def trainable(self, value: bool) -> None: ...
    def __init__(
        self,
        *,
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: DTypeLike | None = None,
        autocast: bool = True,
        name: str | None = None,
        # **kwargs
        input_dim: int | None = None,
        input_shape: Any = None,
    ) -> None: ...

    # *args/**kwargs are allowed, but have obscure footguns and tensorflow documentation discourages their usage.
    # First argument will automatically be cast to layer's compute dtype, but any other tensor arguments will not be.
    # Also various tensorflow tools/apis can misbehave if they encounter a layer with *args/**kwargs.
    def __call__(
        self, inputs: _InputT_contra, *, training: bool = False, mask: TensorCompatible | None = None
    ) -> _OutputT_co: ...
    def call(self, inputs: _InputT_contra, /) -> _OutputT_co: ...

    # input_shape's real type depends on _InputT_contra, but we can't express that without HKT.
    # For example _InputT_contra tf.Tensor -> tf.TensorShape, _InputT_contra dict[str, tf.Tensor] -> dict[str, tf.TensorShape].
    def build(self, input_shape: Any, /) -> None: ...
    @overload
    def compute_output_shape(self: Layer[tf.Tensor, tf.Tensor], input_shape: tf.TensorShape, /) -> tf.TensorShape: ...
    @overload
    def compute_output_shape(self, input_shape: Any, /) -> Any: ...
    def add_weight(
        self,
        shape: Iterable[int | None] | None = None,
        initializer: _Initializer | None = None,
        dtype: DTypeLike | None = None,
        trainable: bool = True,
        autocast: bool = True,
        regularizer: _Regularizer = None,
        constraint: _Constraint = None,
        aggregation: Literal["mean", "sum", "only_first_replica"] = "mean",
        name: str | None = None,
    ) -> tf.Variable: ...
    def add_loss(self, loss: tf.Tensor | Sequence[tf.Tensor] | Callable[[], tf.Tensor]) -> None: ...
    def count_params(self) -> int: ...
    @property
    def trainable_variables(self) -> list[Variable]: ...
    @property
    def non_trainable_variables(self) -> list[Variable]: ...
    @property
    def trainable_weights(self) -> list[Variable]: ...
    @property
    def non_trainable_weights(self) -> list[Variable]: ...
    @property
    def losses(self) -> list[Tensor]: ...
    def get_weights(self) -> list[AnyArray]: ...
    def set_weights(self, weights: Sequence[AnyArray]) -> None: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...
    def __getattr__(self, name: str) -> Incomplete: ...

# Every layer has trainable, dtype, name, and dynamic. At runtime these
# are mainly handled with **kwargs, passed up and then validated.
# In actual implementation there's 12 allowed keyword arguments, but only
# 4 are documented and other 8 are mainly internal. The other 8 can be found
# https://github.com/keras-team/keras/blob/e6784e4302c7b8cd116b74a784f4b78d60e83c26/keras/engine/base_layer.py#L329
# PEP 692 support would be very helpful here and allow removing stubtest allowlist for
# all layer constructors.

# TODO: Replace last Any after adding tf.keras.mixed_precision.Policy.
_LayerDtype: TypeAlias = DTypeLike | dict[str, Any] | Any

_Constraint: TypeAlias = str | dict[str, Any] | Constraint | None

# IndexLookup is not exported by Keras
@type_check_only
class _IndexLookup(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        max_tokens: int | None,
        num_oov_indices: int,
        mask_token: str | None,
        oov_token: str,
        vocabulary_dtype: Literal["int64", "string"],
        vocabulary: str | None | TensorCompatible = None,
        idf_weights: TensorCompatible | None = None,
        invert: bool = False,
        output_mode: Literal["int", "count", "multi_hot", "one_hot", "tf_idf"] = "int",
        sparse: bool = False,
        pad_to_max_tokens: bool = False,
        name: str | None = None,
        *,
        # **kwargs
        vocabulary_size: int | None = None,
        has_input_vocabulary: bool = ...,
        trainable: bool | None = None,
        dtype: _LayerDtype | None = None,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        autocast: bool = True,
    ) -> None: ...
    def compute_output_signature(self, input_spec) -> tf.TensorSpec: ...
    def get_vocabulary(self, include_special_tokens: bool = True) -> list[Incomplete]: ...
    def vocabulary_size(self) -> int: ...

class StringLookup(_IndexLookup):
    def __init__(
        self,
        max_tokens: int | None = None,
        num_oov_indices: int = 1,
        mask_token: str | None = None,
        oov_token: str = "[UNK]",
        vocabulary: str | None | TensorCompatible = None,
        idf_weights: TensorCompatible | None = None,
        invert: bool = False,
        output_mode: Literal["int", "count", "multi_hot", "one_hot", "tf_idf"] = "int",
        pad_to_max_tokens: bool = False,
        sparse: bool = False,
        encoding: str = "utf-8",
        name: str | None = None,
        *,
        # **kwargs passed to IndexLookup
        vocabulary_size: int | None = None,
        has_input_vocabulary: bool = ...,
        trainable: bool | None = None,
        dtype: _LayerDtype | None = None,
        activity_regularizer: _Regularizer = None,
        autocast: bool = True,
    ) -> None: ...
    def adapt(self, data: tf.data.Dataset[TensorLike] | AnyArray | DataSequence, steps: Float | None = None) -> None: ...

class IntegerLookup(_IndexLookup):
    def __init__(
        self,
        max_tokens: int | None = None,
        num_oov_indices: int = 1,
        mask_token: int | None = None,
        oov_token: int = -1,
        vocabulary: str | None | TensorCompatible = None,
        vocabulary_dtype: Literal["int64"] = "int64",
        idf_weights: TensorCompatible | None = None,
        invert: bool = False,
        output_mode: Literal["int", "count", "multi_hot", "one_hot", "tf_idf"] = "int",
        sparse: bool = False,
        pad_to_max_tokens: bool = False,
        name: str | None = None,
        *,
        # **kwargs passed to IndexLookup
        vocabulary_size: int | None = None,
        has_input_vocabulary: bool = ...,
        trainable: bool | None = None,
        dtype: _LayerDtype | None = None,
        activity_regularizer: _Regularizer = None,
        autocast: bool = True,
    ) -> None: ...
    def adapt(self, data: tf.data.Dataset[TensorLike] | AnyArray | DataSequence, steps: Float | None = None) -> None: ...

# Layer's compute_output_shape commonly have instance as first argument name instead of self.
# This is an artifact of actual implementation commonly uses a decorator to define it.
# Layer.build has same weirdness sometimes. For both marked as positional only.
class Dense(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        units: int,
        activation: _Activation = None,
        use_bias: bool = True,
        kernel_initializer: _Initializer = "glorot_uniform",
        bias_initializer: _Initializer = "zeros",
        kernel_regularizer: _Regularizer = None,
        bias_regularizer: _Regularizer = None,
        activity_regularizer: _Regularizer = None,
        kernel_constraint: _Constraint = None,
        bias_constraint: _Constraint = None,
        lora_rank: int | None = None,
        *,
        # **kwargs passed to Layer
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class BatchNormalization(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        axis: int = -1,
        momentum: float = 0.99,
        epsilon: float = 0.001,
        center: bool = True,
        scale: bool = True,
        beta_initializer: _Initializer = "zeros",
        gamma_initializer: _Initializer = "ones",
        moving_mean_initializer: _Initializer = "zeros",
        moving_variance_initializer: _Initializer = "ones",
        beta_regularizer: _Regularizer = None,
        gamma_regularizer: _Regularizer = None,
        beta_constraint: _Constraint = None,
        gamma_constraint: _Constraint = None,
        synchronized: bool = False,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class ReLU(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        max_value: float | None = None,
        negative_slope: float | None = 0.0,
        threshold: float | None = 0.0,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class Dropout(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        rate: float,
        noise_shape: TensorCompatible | Sequence[int | None] | None = None,
        seed: int | None = None,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class Embedding(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        input_dim: int,
        output_dim: int,
        embeddings_initializer: _Initializer = "uniform",
        embeddings_regularizer: _Regularizer = None,
        embeddings_constraint: _Constraint = None,
        mask_zero: bool = False,
        weights=None,
        lora_rank: int | None = None,
        *,
        input_length: int | None = None,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class Conv2D(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        filters: int,
        kernel_size: int | Iterable[int],
        strides: int | Iterable[int] = (1, 1),
        padding: Literal["valid", "same"] = "valid",
        data_format: None | Literal["channels_last", "channels_first"] = None,
        dilation_rate: int | Iterable[int] = (1, 1),
        groups: int = 1,
        activation: _Activation = None,
        use_bias: bool = True,
        kernel_initializer: _Initializer = "glorot_uniform",
        bias_initializer: _Initializer = "zeros",
        kernel_regularizer: _Regularizer = None,
        bias_regularizer: _Regularizer = None,
        activity_regularizer: _Regularizer = None,
        kernel_constraint: _Constraint = None,
        bias_constraint: _Constraint = None,
        *,
        # **kwargs passed to Layer
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

Convolution2D = Conv2D

class Identity(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class LayerNormalization(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        axis: int = -1,
        epsilon: float = 0.001,
        center: bool = True,
        scale: bool = True,
        rms_scaling: bool = False,
        beta_initializer: _Initializer = "zeros",
        gamma_initializer: _Initializer = "ones",
        beta_regularizer: _Regularizer = None,
        gamma_regularizer: _Regularizer = None,
        beta_constraint: _Constraint = None,
        gamma_constraint: _Constraint = None,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

class MultiHeadAttention(Layer[Any, tf.Tensor]):
    def __init__(
        self,
        num_heads: int,
        key_dim: int | None,
        value_dim: int | None = None,
        dropout: float = 0.0,
        use_bias: bool = True,
        output_shape: tuple[int, ...] | None = None,
        attention_axes: tuple[int, ...] | None = None,
        kernel_initializer: _Initializer = "glorot_uniform",
        bias_initializer: _Initializer = "zeros",
        kernel_regularizer: Regularizer | None = None,
        bias_regularizer: _Regularizer | None = None,
        activity_regularizer: _Regularizer | None = None,
        kernel_constraint: _Constraint | None = None,
        bias_constraint: _Constraint | None = None,
        seed: int | None = None,
        *,
        # **kwargs passed to Layer
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...
    @overload  # type: ignore[override]
    def __call__(
        self,
        query: tf.Tensor,
        value: tf.Tensor,
        key: tf.Tensor | None,
        attention_mask: tf.Tensor | None,
        return_attention_scores: Literal[False],
        training: bool,
        use_causal_mask: bool,
    ) -> tf.Tensor: ...
    @overload
    def __call__(
        self,
        query: tf.Tensor,
        value: tf.Tensor,
        key: tf.Tensor | None,
        attention_mask: tf.Tensor | None,
        return_attention_scores: Literal[True],
        training: bool,
        use_causal_mask: bool,
    ) -> tuple[tf.Tensor, tf.Tensor]: ...
    @overload
    def __call__(
        self,
        query: tf.Tensor,
        value: tf.Tensor,
        key: tf.Tensor | None = None,
        attention_mask: tf.Tensor | None = None,
        return_attention_scores: bool = False,
        training: bool = False,
        use_causal_mask: bool = False,
    ) -> tuple[tf.Tensor, tf.Tensor] | tf.Tensor: ...

class GaussianDropout(Layer[tf.Tensor, tf.Tensor]):
    def __init__(
        self,
        rate: float,
        seed: int | None = None,
        *,
        # **kwargs passed to Layer
        activity_regularizer: _Regularizer = None,
        trainable: bool = True,
        dtype: _LayerDtype | None = None,
        autocast: bool = True,
        name: str | None = None,
    ) -> None: ...

def __getattr__(name: str): ...  # incomplete module



from collections.abc import Callable, Hashable, MutableMapping, Sequence
from typing import Any

import branca  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
import folium  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
import pandas as pd
import xyzservices  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
from matplotlib.colors import Colormap  # type: ignore[import-not-found]
from numpy.typing import ArrayLike, NDArray

from .geodataframe import GeoDataFrame
from .geoseries import GeoSeries

def _explore(
    df: GeoDataFrame,
    column: Hashable | NDArray[Any] | pd.Series[Any] | None = None,  # Accepts "any" array or series
    cmap: str | Colormap | branca.colormap.ColorMap | Sequence[str] | Callable[[Any], str] | None = None,  # accepts "any" object
    color: str | ArrayLike | None = None,
    m: folium.Map | None = None,
    tiles: str | folium.TileLayer | xyzservices.TileProvider | None = "OpenStreetMap",
    attr: str | None = None,
    tooltip: bool = True,
    popup: bool = False,
    highlight: bool = True,
    categorical: bool = False,
    legend: bool = True,
    scheme: str | None = None,
    k: int = 5,
    vmin: float | None = None,
    vmax: float | None = None,
    width: float | str = "100%",
    height: float | str = "100%",
    categories: Sequence[Any] | NDArray[Any] | pd.Series[Any] | pd.Index[Any] | None = None,  # categories can have "any" type
    classification_kwds: MutableMapping[str, Any] | None = None,
    control_scale: bool = True,
    marker_type: str | folium.Marker | None = None,
    # The following kwds will never be typed more precisely than "Any"
    marker_kwds: MutableMapping[str, Any] = {},
    style_kwds: MutableMapping[str, Any] = {},
    highlight_kwds: MutableMapping[str, Any] = {},
    missing_kwds: MutableMapping[str, Any] = {},
    tooltip_kwds: MutableMapping[str, Any] = {},
    popup_kwds: MutableMapping[str, Any] = {},
    legend_kwds: MutableMapping[str, Any] = {},
    map_kwds: MutableMapping[str, Any] = {},
    **kwargs,
) -> folium.Map: ...
def _explore_geoseries(
    s: GeoSeries,
    color: str | ArrayLike | None = None,
    m: folium.Map | None = None,
    tiles: str | folium.TileLayer | xyzservices.TileProvider | None = "OpenStreetMap",
    attr: str | None = None,
    highlight: bool = True,
    width: float | str = "100%",
    height: float | str = "100%",
    control_scale: bool = True,
    marker_type: str | folium.Marker | None = None,
    # The following kwds will never be typed more precisely than "Any"
    marker_kwds: MutableMapping[str, Any] = {},
    style_kwds: MutableMapping[str, Any] = {},
    highlight_kwds: MutableMapping[str, Any] = {},
    map_kwds: MutableMapping[str, Any] = {},
    **kwargs,
) -> folium.Map: ...



from collections.abc import Iterator, Sequence
from configparser import ConfigParser
from re import Pattern
from typing import Any
from typing_extensions import Self

entry_point_pattern: Pattern[str]
file_in_zip_pattern: Pattern[str]

class BadEntryPoint(Exception):
    epstr: str
    def __init__(self, epstr: str) -> None: ...
    @staticmethod
    def err_to_warnings() -> Iterator[None]: ...

class NoSuchEntryPoint(Exception):
    group: str
    name: str
    def __init__(self, group: str, name: str) -> None: ...

class CaseSensitiveConfigParser(ConfigParser): ...

class EntryPoint:
    name: str
    module_name: str
    object_name: str
    extras: Sequence[str] | None
    distro: Distribution | None
    def __init__(
        self, name: str, module_name: str, object_name: str, extras: Sequence[str] | None = ..., distro: Distribution | None = ...
    ) -> None: ...
    def load(self) -> Any: ...
    @classmethod
    def from_string(cls, epstr: str, name: str, distro: Distribution | None = ...) -> Self: ...

class Distribution:
    name: str
    version: str
    def __init__(self, name: str, version: str) -> None: ...
    @classmethod
    def from_name_version(cls, name: str) -> Self: ...

def iter_files_distros(
    path: Sequence[str] | None = ..., repeated_distro: str = ...
) -> Iterator[tuple[ConfigParser, Distribution | None]]: ...
def get_single(group: str, name: str, path: Sequence[str] | None = ...) -> EntryPoint: ...
def get_group_named(group: str, path: Sequence[str] | None = ...) -> dict[str, EntryPoint]: ...
def get_group_all(group: str, path: Sequence[str] | None = ...) -> list[EntryPoint]: ...



class XDG:
    def __init__(self) -> None: ...
    @property
    def XDG_DATA_HOME(self) -> str: ...
    @property
    def XDG_CONFIG_HOME(self) -> str: ...
    @property
    def XDG_CACHE_HOME(self) -> str: ...
    @property
    def XDG_RUNTIME_DIR(self) -> str: ...
    @property
    def XDG_DATA_DIRS(self) -> tuple[str, ...]: ...
    @property
    def XDG_CONFIG_DIRS(self) -> tuple[str, ...]: ...

class XDGPackage(XDG):
    def __init__(self, package_name: str) -> None: ...
    @property
    def XDG_DATA_HOME(self) -> str: ...
    @property
    def XDG_CONFIG_HOME(self) -> str: ...
    @property
    def XDG_CACHE_HOME(self) -> str: ...
    @property
    def XDG_RUNTIME_DIR(self) -> str: ...

class XDGPedanticPackage(XDGPackage):
    @property
    def XDG_DATA_HOME(self) -> str: ...
    @property
    def XDG_CONFIG_HOME(self) -> str: ...
    @property
    def XDG_CACHE_HOME(self) -> str: ...
    @property
    def XDG_RUNTIME_DIR(self) -> str: ...



from collections.abc import Callable
from typing import TypeVar
from typing_extensions import ParamSpec

_P = ParamSpec("_P")
_R = TypeVar("_R")

def check_resource(resource_name: str) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]: ...
def minimum_version(version: str) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]: ...
def update_headers(f: Callable[_P, _R]) -> Callable[_P, _R]: ...



import os
import sys

sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__))))

from . import pygls
from . import lark
from . import lsprotocol
from . import pluggy

__all__ = ["pygls", "lark", "pluggy", "lsprotocol"]



from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

def asNative(s): ...

EXTENSIONS: Incomplete
FF_FIXED: Incomplete
FF_SERIF: Incomplete
FF_SYMBOLIC: Incomplete
FF_SCRIPT: Incomplete
FF_NONSYMBOLIC: Incomplete
FF_ITALIC: Incomplete
FF_ALLCAP: Incomplete
FF_SMALLCAP: Incomplete
FF_FORCEBOLD: Incomplete

class FontDescriptor:
    name: Incomplete
    fullName: Incomplete
    familyName: Incomplete
    styleName: Incomplete
    isBold: bool
    isItalic: bool
    isFixedPitch: bool
    isSymbolic: bool
    typeCode: Incomplete
    fileName: Incomplete
    metricsFileName: Incomplete
    timeModified: int
    def __init__(self) -> None: ...
    def getTag(self): ...

class FontFinder:
    useCache: Incomplete
    validate: Incomplete
    verbose: Incomplete
    def __init__(
        self, dirs=[], useCache: bool = True, validate: bool = False, recur: bool = False, fsEncoding=None, verbose: int = 0
    ) -> None: ...
    def addDirectory(self, dirName, recur=None) -> None: ...
    def addDirectories(self, dirNames, recur=None) -> None: ...
    def getFamilyNames(self): ...
    def getFontsInFamily(self, familyName): ...
    def getFamilyXmlReport(self): ...
    def getFontsWithAttributes(self, **kwds): ...
    def getFont(self, familyName, bold: bool = False, italic: bool = False): ...
    def save(self, fileName) -> None: ...
    def load(self, fileName) -> None: ...
    def search(self) -> None: ...

def test() -> None: ...



from _typeshed import Incomplete, SupportsItems
from collections.abc import Callable, Collection, Hashable, Iterable, Mapping, Sequence
from typing import Any, Generic, Literal, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias, Unpack

import numpy as np
from matplotlib.axes import Axes  # type: ignore[import-not-found]
from matplotlib.collections import LineCollection, PathCollection  # type: ignore[import-not-found]
from matplotlib.colors import Colormap  # type: ignore[import-not-found]
from matplotlib.patches import FancyArrowPatch  # type: ignore[import-not-found]
from matplotlib.text import Text  # type: ignore[import-not-found]
from matplotlib.typing import ColorType  # type: ignore[import-not-found]
from networkx._typing import Array2D
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node

__all__ = [
    "display",
    "apply_matplotlib_colors",
    "draw",
    "draw_networkx",
    "draw_networkx_nodes",
    "draw_networkx_edges",
    "draw_networkx_labels",
    "draw_networkx_edge_labels",
    "draw_bipartite",
    "draw_circular",
    "draw_kamada_kawai",
    "draw_random",
    "draw_spectral",
    "draw_spring",
    "draw_planar",
    "draw_shell",
    "draw_forceatlas2",
]

_G = TypeVar("_G", bound=Graph[Any])

# types from matplotlib
_FontSize: TypeAlias = Literal["xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"] | float
_FontWeight: TypeAlias = (
    Literal[
        "ultralight",
        "light",
        "normal",
        "regular",
        "book",
        "medium",
        "roman",
        "semibold",
        "demibold",
        "demi",
        "bold",
        "heavy",
        "extra bold",
        "black",
    ]
    | int
)
_HAlign: TypeAlias = Literal["left", "center", "right"]
_VAlign: TypeAlias = Literal["baseline", "bottom", "center", "center_baseline", "top"]

@type_check_only
class _DrawNetworkxKwds(TypedDict, Generic[_Node], total=False):
    # draw nodes keywords; keep in sync with draw_networkx_nodes
    node_color: ColorType | Collection[ColorType] | Collection[float]
    cmap: str | Colormap | None
    vmin: float | None
    vmax: float | None
    linewidths: float | Collection[float] | None
    edgecolors: Literal["face", "none"] | ColorType | Collection[ColorType] | Collection[float] | None
    margins: float | tuple[float, float] | None
    # draw edges keywords; keep in sync with draw_networkx_edges
    edgelist: Collection[_Node | Hashable] | None  # (u, v, k) for multigraphs and (u, v) for simple graphs
    width: float | Collection[float]
    edge_color: ColorType | Collection[ColorType]
    style: str | Collection[str]
    arrowstyle: str | Collection[str] | None
    arrowsize: float | list[int] | list[float]
    edge_cmap: Colormap | None
    edge_vmin: float | None
    edge_vmax: float | None
    connectionstyle: str | Iterable[str]
    min_source_margin: int | Collection[int]
    min_target_margin: int | Collection[int]
    # draw labels keywords; keep in sync with draw_networkx_labels
    labels: Mapping[_Node, object] | None
    font_size: _FontSize | Mapping[_Node, _FontSize]
    font_color: ColorType | Mapping[_Node, Colormap]
    font_family: str | Mapping[_Node, str]
    font_weight: _FontWeight | Mapping[_Node, _FontWeight]
    bbox: dict[str, Any] | None
    horizontalalignment: _HAlign
    verticalalignment: _VAlign
    clip_on: bool
    # common keywords
    nodelist: Sequence[_Node] | None
    node_size: float | Collection[float]
    node_shape: str
    alpha: float | Collection[float] | None
    label: str | None
    hide_ticks: bool

def apply_matplotlib_colors(
    G: Graph[_Node],
    src_attr: str,
    dest_attr: str,
    map: str | Colormap,
    vmin: float | None = None,
    vmax: float | None = None,
    nodes: bool = True,
) -> None: ...
def display(
    G: _G,
    canvas: Axes | None = None,
    *,
    pos: str | Callable[[_G], Mapping[_Node, Collection[float]]] = ...,
    node_visible: str | bool = ...,
    node_color: str = ...,
    node_size: str | float = ...,
    node_label: str | bool = ...,
    node_shape: str = ...,
    node_alpha: str = ...,
    node_border_width: str = ...,
    node_border_color: str = ...,
    edge_visible: str | bool = ...,
    edge_width: str | int = ...,
    edge_color: str | ColorType = ...,
    edge_label: str = ...,
    edge_style: str = ...,
    edge_alpha: str | float = ...,
    arrowstyle: str = ...,
    arrowsize: str | int = ...,
    edge_curvature: str = ...,
    edge_source_margin: str | int = ...,
    edge_target_margin: str | int = ...,
    hide_ticks: bool = True,
) -> _G: ...
def draw(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]] | None = None,
    ax: Axes | None = None,
    *,
    with_labels: bool = ...,  # default depends on whether a label argument is passed
    **kwds: Unpack[_DrawNetworkxKwds[_Node]],
) -> None: ...
def draw_networkx(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]] | None = None,
    arrows: bool | None = None,
    with_labels: bool = True,
    *,
    ax: Axes | None = None,
    **kwds: Unpack[_DrawNetworkxKwds[_Node]],
) -> None: ...
def draw_networkx_nodes(  # keep in sync with _DrawNetworkxKwds above
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    nodelist: Collection[_Node] | None = None,
    node_size: float | Collection[float] = 300,
    node_color: ColorType | Collection[ColorType] | Collection[float] = "#1f78b4",
    node_shape: str = "o",
    alpha: float | Collection[float] | None = None,
    cmap: str | Colormap | None = None,
    vmin: float | None = None,
    vmax: float | None = None,
    ax: Axes | None = None,
    linewidths: float | Collection[float] | None = None,
    edgecolors: Literal["face", "none"] | ColorType | Collection[ColorType] | Collection[float] | None = None,
    label: str | None = None,
    margins: float | tuple[float, float] | None = None,
    hide_ticks: bool = True,
) -> PathCollection: ...
@overload  # arrows=None -> LineCollection if G is undirected, list[FancyArrowPatch] if G is directed
def draw_networkx_edges(  # keep in sync with _DrawNetworkxKwds above
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    edgelist: Collection[_Node | Hashable] | None = None,  # (u, v, k) for multigraphs and (u, v) for simple graphs
    width: float | Collection[float] = 1.0,
    edge_color: ColorType | Collection[ColorType] = "k",
    style: str | Collection[str] = "solid",
    alpha: float | Collection[float] | None = None,
    arrowstyle: str | Collection[str] | None = None,
    arrowsize: float | list[int] | list[float] = 10,  # documented as int, mpl accepts float
    edge_cmap: Colormap | None = None,
    edge_vmin: float | None = None,
    edge_vmax: float | None = None,
    ax: Axes | None = None,
    arrows: None = None,
    label: str | None = None,  # documented as str, mpl accepts any object as it calls str on it
    node_size: float | Collection[float] = 300,
    nodelist: Sequence[_Node] | None = None,
    node_shape: str = "o",
    connectionstyle: str | Iterable[str] = "arc3",
    min_source_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    min_target_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    hide_ticks: bool = True,
) -> LineCollection | list[FancyArrowPatch]: ...
@overload  # directed graph and arrows=None -> list[FancyArrowPatch]
def draw_networkx_edges(
    G: DiGraph[_Node],
    pos: Mapping[_Node, Collection[float]],
    edgelist: Collection[_Node | Hashable] | None = None,  # (u, v, k) for multigraphs and (u, v) for simple graphs
    width: float | Collection[float] = 1.0,
    edge_color: ColorType | Collection[ColorType] = "k",
    style: str | Collection[str] = "solid",
    alpha: float | Collection[float] | None = None,
    arrowstyle: str | Collection[str] | None = None,
    arrowsize: float | list[int] | list[float] = 10,  # documented as int, mpl accepts float
    edge_cmap: Colormap | None = None,
    edge_vmin: float | None = None,
    edge_vmax: float | None = None,
    ax: Axes | None = None,
    arrows: None = None,
    label: str | None = None,  # documented as str, mpl accepts any object as it calls str on it
    node_size: float | Collection[float] = 300,
    nodelist: Sequence[_Node] | None = None,
    node_shape: str = "o",
    connectionstyle: str | Iterable[str] = "arc3",
    min_source_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    min_target_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    hide_ticks: bool = True,
) -> list[FancyArrowPatch]: ...
@overload  # arrows=True -> list[FancyArrowPatch]
def draw_networkx_edges(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    edgelist: Collection[_Node | Hashable] | None = None,  # (u, v, k) for multigraphs and (u, v) for simple graphs
    width: float | Collection[float] = 1.0,
    edge_color: ColorType | Collection[ColorType] = "k",
    style: str | Collection[str] = "solid",
    alpha: float | Collection[float] | None = None,
    arrowstyle: str | Collection[str] | None = None,
    arrowsize: float | list[int] | list[float] = 10,  # documented as int, mpl accepts float
    edge_cmap: Colormap | None = None,
    edge_vmin: float | None = None,
    edge_vmax: float | None = None,
    ax: Axes | None = None,
    *,
    arrows: Literal[True],
    label: str | None = None,  # documented as str, mpl accepts any object as it calls str on it
    node_size: float | Collection[float] = 300,
    nodelist: Sequence[_Node] | None = None,
    node_shape: str = "o",
    connectionstyle: str | Iterable[str] = "arc3",
    min_source_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    min_target_margin: int | Collection[int] = 0,  # documented as int, mpl accepts float
    hide_ticks: bool = True,
) -> list[FancyArrowPatch]: ...
@overload  # arrows=False -> LineCollection
def draw_networkx_edges(
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    edgelist: Collection[_Node | Hashable] | None = None,  # (u, v, k) for multigraphs and (u, v) for simple graphs
    width: float | Collection[float] = 1.0,
    edge_color: ColorType | Collection[ColorType] = "k",
    style: str | Collection[str] = "solid",
    alpha: float | Collection[float] | None = None,
    *,
    edge_cmap: Colormap | None = None,
    edge_vmin: float | None = None,
    edge_vmax: float | None = None,
    ax: Axes | None = None,
    arrows: Literal[False],
    label: str | None = None,  # documented as str, mpl accepts any object as it calls str on it
    node_size: float | Collection[float] = 300,
    nodelist: Sequence[_Node] | None = None,
    node_shape: str = "o",
    hide_ticks: bool = True,
) -> LineCollection: ...
def draw_networkx_labels(  # keep in sync with _DrawNetworkxKwds above
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    labels: Mapping[_Node, object] | None = None,  # labels are explicitly converted to str
    font_size: _FontSize | Mapping[_Node, _FontSize] = 12,
    font_color: ColorType | Mapping[_Node, Colormap] = "k",
    font_family: str | Mapping[_Node, str] = "sans-serif",
    font_weight: _FontWeight | Mapping[_Node, _FontWeight] = "normal",
    alpha: float | Mapping[_Node, float] | None = None,
    bbox: dict[str, Any] | None = None,  # Any comes from mpl
    horizontalalignment: _HAlign = "center",  # doc is wrong, doesn't really accept array
    verticalalignment: _VAlign = "center",  # doc is wrong, doesn't really accept array
    ax: Axes | None = None,
    clip_on: bool = True,
    hide_ticks: bool = True,
) -> dict[_Node, Text]: ...
def draw_networkx_edge_labels(
    # TODO: find a way to have a covariant list for params annotated with `something | list[Incomplete]`
    G: Graph[_Node],
    pos: Mapping[_Node, Collection[float]],
    edge_labels: (
        SupportsItems[
            Collection[_Node | Hashable],  # (u, v, k) for multigraphs and (u, v) for simple graphs
            object,  # labels are explicitly converted to str and nx internally passes non-str
        ]
        | None
    ) = None,
    label_pos: float | list[Incomplete] = 0.5,
    font_size: _FontSize | list[Incomplete] = 10,
    font_color: ColorType | list[Incomplete] = "k",
    font_family: str = "sans-serif",
    font_weight: _FontWeight | list[Incomplete] = "normal",
    alpha: float | list[Incomplete] | None = None,
    bbox: dict[str, Any] | None = None,  # Any comes from mpl
    horizontalalignment: _HAlign | list[Incomplete] = "center",
    verticalalignment: _VAlign | list[Incomplete] = "center",
    ax: Axes | None = None,
    rotate: bool | list[bool] = True,
    clip_on: bool = True,
    node_size: float | Collection[float] = 300,
    nodelist: Sequence[_Node] | None = None,
    connectionstyle: str | Iterable[str] = "arc3",
    hide_ticks: bool = True,
) -> dict[tuple[_Node, _Node] | tuple[_Node, _Node, Any], Text]: ...  # Any is for multigraph key
def draw_bipartite(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_circular(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_kamada_kawai(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_random(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_spectral(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_spring(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_shell(
    G: Graph[_Node],
    nlist: Collection[Collection[_Node]] | None = None,
    *,
    ax: Axes | None = None,
    with_labels: bool = ...,
    **kwargs: Unpack[_DrawNetworkxKwds[_Node]],
) -> None: ...
def draw_planar(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def draw_forceatlas2(
    G: Graph[_Node], *, ax: Axes | None = None, with_labels: bool = ..., **kwargs: Unpack[_DrawNetworkxKwds[_Node]]
) -> None: ...
def apply_alpha(
    colors: ColorType | Collection[ColorType] | Collection[float],
    alpha: float | Collection[float],
    elem_list: Collection[object],  # nx objects (nodes, edges, labels) but its content is not used!
    cmap: str | Colormap | None = None,
    vmin: float | None = None,
    vmax: float | None = None,
) -> Array2D[np.float64]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FullTypeId:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FullTypeIdEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FullTypeId.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    TFT_UNSET: _FullTypeId.ValueType  # 0
    """The default represents an uninitialized values."""
    TFT_VAR: _FullTypeId.ValueType  # 1
    """Type symbols. Used to construct more complex type expressions like
    algebraic data types.

    Type variables may serve as placeholder for any other type ID in type
    templates.

    Examples:
      TFT_DATASET[TFT_VAR["T"]] is a Dataset returning a type indicated by "T".
      TFT_TENSOR[TFT_VAR["T"]] is a Tensor of n element type indicated by "T".
      TFT_TENSOR[TFT_VAR["T"]], TFT_TENSOR[TFT_VAR["T"]] are two tensors of
        identical element types.
      TFT_TENSOR[TFT_VAR["P"]], TFT_TENSOR[TFT_VAR["Q"]] are two tensors of
        independent element types.
    """
    TFT_ANY: _FullTypeId.ValueType  # 2
    """Wildcard type. Describes a parameter of unknown type. In TensorFlow, that
    can mean either a "Top" type (accepts any type), or a dynamically typed
    object whose type is unknown in context.
    Important: "unknown" does not necessarily mean undeterminable!
    """
    TFT_PRODUCT: _FullTypeId.ValueType  # 3
    """The algebraic product type. This is an algebraic type that may be used just
    for logical grouping. Not to confused with TFT_TUPLE which describes a
    concrete object of several elements.

    Example:
      TFT_DATASET[TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT64]]]
        is a Dataset producing two tensors, an integer one and a float one.
    """
    TFT_NAMED: _FullTypeId.ValueType  # 4
    """Represents a named field, with the name stored in the attribute.

    Parametrization:
      TFT_NAMED[<type>]{<name>}
      * <type> is the type of the field
      * <name> is the field name, as string (thpugh can theoretically be an int
        as well)

    Example:
      TFT_RECORD[
        TFT_NAMED[TFT_TENSOR[TFT_INT32]]{'foo'},
        TFT_NAMED[TFT_TENSOR[TFT_FLOAT32]]{'bar'},
      ]
        is a structure with two fields, an int tensor "foo" and a float tensor
        "bar".
    """
    TFT_FOR_EACH: _FullTypeId.ValueType  # 20
    """Template definition. Expands the variables by repeating a template as
    arguments of container.

    Parametrization:
      TFT_FOR_EACH[<container_type>, <template>, <expansions>]
      * <container_type> is the type of the container that the template will be
        expanded into
      * <template> is any type definition that potentially contains type
        variables
      * <expansions> is a TFT_VAR and may include more types in the future

    Example:
      TFT_FOR_EACH[
            TFT_PRODUCT,
            TFT_TENSOR[TFT_VAR["t"]],
            TFT_VAR["t"]
        ]
        will substitute a T = TFT_INT32 to TFT_PRODUCT[TFT_TENSOR[TFT_INT32]]
        and a T = (TFT_INT32, TFT_INT64) to
        TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_INT64]].
    """
    TFT_CALLABLE: _FullTypeId.ValueType  # 100
    """Callable types describe functions and ops.

    Parametrization:
      TFT_CALLABLE[<arg type>, <return type>]
      * <arg type> is the type of the arguments; TFT_PRODUCT represents
      multiple
        arguments.
      * <return type> is the return type; TFT_PRODUCT represents multiple
        return values (that means that callables returning multiple things
        don't necessarily return a single tuple).

    Example:
      TFT_CALLABLE[
        TFT_ANY,
        TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT64]],
      ]
        is a callable with unspecified (for now) input arguments, and
        two return values of type tensor.
    """
    TFT_TENSOR: _FullTypeId.ValueType  # 1000
    """Concrete type IDs, representing "proper" data types that can describe
    runtime TensorFlow objects.

    The usual Tensor. This is a parametric type.

    Parametrization:
      TFT_TENSOR[<element type>, <shape type>]
      * <element type> is currently limited to one of the element types
        defined below.
      * <shape type> is not yet defined, and may only be TFT_UNKNOWN for now.

    A TFT_SHAPE type will be defined in the future.

    Example:
      TFT_TENSOR[TFT_INT32, TFT_UNKNOWN]
        is a Tensor of int32 element type and unknown shape.

    TODO(mdan): Define TFT_SHAPE and add more examples.
    """
    TFT_ARRAY: _FullTypeId.ValueType  # 1001
    """Array (or tensorflow::TensorList in the variant type registry).
    Note: this is not to be confused with the deprecated `TensorArray*` ops
    which are not supported by FullType.
    This type represents a random-access list whose elements can be
    described by a single type. Although immutable, Array is expected to
    support efficient mutation semantics (i.e. element update) in the
    user-facing API.
    The element type may be generic or even TFT_ANY for a heterogenous list.

    Parametrization:
      TFT_ARRAY[<element type>]
      * <element type> may be any concrete type.

    Examples:
      TFT_ARRAY[TFT_TENSOR[TFT_INT32]] is a TensorArray holding int32 Tensors
        of any shape.
      TFT_ARRAY[TFT_TENSOR[TFT_UNKNOWN]] is a TensorArray holding Tensors of
        mixed element types.
      TFT_ARRAY[TFT_UNKNOWN] is a TensorArray holding any element type.
      TFT_ARRAY[] is equivalent to TFT_ARRAY[TFT_UNKNOWN].
      TFT_ARRAY[TFT_ARRAY[]] is an array or arrays (of unknown types).
    """
    TFT_OPTIONAL: _FullTypeId.ValueType  # 1002
    """Optional (or tensorflow::OptionalVariant in the variant type registry).
    This type represents a value that may either hold an element of a single
    specified type, or nothing at all.

    Parametrization:
      TFT_OPTIONAL[<element type>]
      * <element type> may be any concrete type.

    Examples:
      TFT_OPTIONAL[TFT_TENSOR[TFT_INT32]] is an Optional holding an int32
        Tensor of any shape.
    """
    TFT_LITERAL: _FullTypeId.ValueType  # 1003
    """Literal types describe compile-time constant values.
    Literal types may also participate in dependent types.

    Parametrization:
      TFT_LITERAL[<value type>]{<value>}
      * <value type> may be any concrete type compatible that can hold <value>
      * <value> is the type's attribute, and holds the actual literal value

    Examples:
      TFT_LITERAL[TFT_INT32]{1} is the compile-time constant 1.
    """
    TFT_ENCODED: _FullTypeId.ValueType  # 1004
    """Encoding types describe a value of a certain type, encoded as a different
    type.

    Parametrization:
      TFT_ENCODED[<encoded type>, <encoding type>]
      * <encoded type> may be any type
      * <encoding type> may be any type

    Examples:
      TFT_ENCODING[TFT_INT32, TFT_STRING] is an integer encoded as string.
    """
    TFT_SHAPE_TENSOR: _FullTypeId.ValueType  # 1005
    """The type of "shape tensors" where the runtime value is the shape of
    some tensor(s), i.e. the output of tf.shape.
    Shape tensors have special, host-only placement, in contrast to
    TFT_TENSOR[TFT_INT32] which is the type of a normal numeric tensor
    with no special placement.

    Examples:
      TFT_SHAPE_TENSOR[TFT_INT32] is the most common
      TFT_SHAPE_TENSOR[TFT_INT64] is also allowed
    """
    TFT_BOOL: _FullTypeId.ValueType  # 200
    """Type attributes. These always appear in the parametrization of a type,
    never alone. For example, there is no such thing as a "bool" TensorFlow
    object (for now).

    The bool element type.
    TODO(mdan): Quantized types, legacy representations (e.g. ref)
    """
    TFT_UINT8: _FullTypeId.ValueType  # 201
    """Integer element types."""
    TFT_UINT16: _FullTypeId.ValueType  # 202
    TFT_UINT32: _FullTypeId.ValueType  # 203
    TFT_UINT64: _FullTypeId.ValueType  # 204
    TFT_INT8: _FullTypeId.ValueType  # 205
    TFT_INT16: _FullTypeId.ValueType  # 206
    TFT_INT32: _FullTypeId.ValueType  # 207
    TFT_INT64: _FullTypeId.ValueType  # 208
    TFT_HALF: _FullTypeId.ValueType  # 209
    """Floating-point element types."""
    TFT_FLOAT: _FullTypeId.ValueType  # 210
    TFT_DOUBLE: _FullTypeId.ValueType  # 211
    TFT_BFLOAT16: _FullTypeId.ValueType  # 215
    TFT_COMPLEX64: _FullTypeId.ValueType  # 212
    """Complex element types.
    TODO(mdan): Represent as TFT_COMPLEX[TFT_DOUBLE] instead?
    """
    TFT_COMPLEX128: _FullTypeId.ValueType  # 213
    TFT_STRING: _FullTypeId.ValueType  # 214
    """The string element type."""
    TFT_DATASET: _FullTypeId.ValueType  # 10102
    """Other types that we don't know yet whether they will become part of the
    core type system or be consisdered third-party (and consequently moved to
    user-defined type mechanisms). Presently, they are effectively in the core
    type system, because key compilation passes like Placer account for their
    existence.

    Datasets created by tf.data ops and APIs. Datasets have generator/iterable
    semantics, that is, one can construct an iterator from them. Like
    Array, they are considered to return elements that can be described
    by a single type. Unlike Array, they do not support random access or
    mutation, and can potentially produce an infinite number of elements.
    A datasets can produce logical structures (e.g. multiple elements). This
    is expressed using TFT_PRODUCT.


    Parametrization: TFT_DATASET[<element type>].
      * <element type> may be a concrete type or a type symbol. It represents
        the data type of the elements produced by the dataset.

    Examples:
      TFT_DATSET[TFT_TENSOR[TFT_INT32]] is a Dataset producing single int32
        Tensors of unknown shape.
      TFT_DATSET[TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT32]] is
        a Dataset producing pairs of Tensors, one integer and one float.
    Note: The high ID number is to prepare for the eventuality that Datasets
    will be supported by user types in the future.
    """
    TFT_RAGGED: _FullTypeId.ValueType  # 10103
    """A ragged tensor created by tf.ragged ops and APIs.

    Parametrization: TFT_RAGGED[<element_type>].
    """
    TFT_ITERATOR: _FullTypeId.ValueType  # 10104
    """Iterators created by tf.data ops and APIs. Very similar to Datasets, except
    they are mutable.


    Parametrization: TFT_ITERATOR[<element type>].
      * <element type> may be a concrete type or a type symbol. It represents
        the data type of the elements produced by the dataset.
    """
    TFT_MUTEX_LOCK: _FullTypeId.ValueType  # 10202
    """A mutex lock tensor, produced by tf.raw_ops.MutexLock.
    Unlike strict execution models, where ownership of a lock is denoted by
    "running after the lock has been acquired", in non-strict mode, lock
    ownership is in the true sense: "the op argument representing the lock is
    available".
    Mutex locks are the dynamic counterpart of control dependencies.
    TODO(mdan): Properly document this thing.

    Parametrization: TFT_MUTEX_LOCK[].
    """
    TFT_LEGACY_VARIANT: _FullTypeId.ValueType  # 10203
    """The equivalent of a Tensor with DT_VARIANT dtype, kept here to simplify
    translation. This type should not normally appear after type inference.
    Note that LEGACY_VARIANT != ANY: TENSOR[INT32] is a subtype of ANY, but is
    not a subtype of LEGACY_VARIANT.
    """

class FullTypeId(_FullTypeId, metaclass=_FullTypeIdEnumTypeWrapper):
    """LINT.IfChange
    Experimental. Represents the complete type information of a TensorFlow value.
    """

TFT_UNSET: FullTypeId.ValueType  # 0
"""The default represents an uninitialized values."""
TFT_VAR: FullTypeId.ValueType  # 1
"""Type symbols. Used to construct more complex type expressions like
algebraic data types.

Type variables may serve as placeholder for any other type ID in type
templates.

Examples:
  TFT_DATASET[TFT_VAR["T"]] is a Dataset returning a type indicated by "T".
  TFT_TENSOR[TFT_VAR["T"]] is a Tensor of n element type indicated by "T".
  TFT_TENSOR[TFT_VAR["T"]], TFT_TENSOR[TFT_VAR["T"]] are two tensors of
    identical element types.
  TFT_TENSOR[TFT_VAR["P"]], TFT_TENSOR[TFT_VAR["Q"]] are two tensors of
    independent element types.
"""
TFT_ANY: FullTypeId.ValueType  # 2
"""Wildcard type. Describes a parameter of unknown type. In TensorFlow, that
can mean either a "Top" type (accepts any type), or a dynamically typed
object whose type is unknown in context.
Important: "unknown" does not necessarily mean undeterminable!
"""
TFT_PRODUCT: FullTypeId.ValueType  # 3
"""The algebraic product type. This is an algebraic type that may be used just
for logical grouping. Not to confused with TFT_TUPLE which describes a
concrete object of several elements.

Example:
  TFT_DATASET[TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT64]]]
    is a Dataset producing two tensors, an integer one and a float one.
"""
TFT_NAMED: FullTypeId.ValueType  # 4
"""Represents a named field, with the name stored in the attribute.

Parametrization:
  TFT_NAMED[<type>]{<name>}
  * <type> is the type of the field
  * <name> is the field name, as string (thpugh can theoretically be an int
    as well)

Example:
  TFT_RECORD[
    TFT_NAMED[TFT_TENSOR[TFT_INT32]]{'foo'},
    TFT_NAMED[TFT_TENSOR[TFT_FLOAT32]]{'bar'},
  ]
    is a structure with two fields, an int tensor "foo" and a float tensor
    "bar".
"""
TFT_FOR_EACH: FullTypeId.ValueType  # 20
"""Template definition. Expands the variables by repeating a template as
arguments of container.

Parametrization:
  TFT_FOR_EACH[<container_type>, <template>, <expansions>]
  * <container_type> is the type of the container that the template will be
    expanded into
  * <template> is any type definition that potentially contains type
    variables
  * <expansions> is a TFT_VAR and may include more types in the future

Example:
  TFT_FOR_EACH[
        TFT_PRODUCT,
        TFT_TENSOR[TFT_VAR["t"]],
        TFT_VAR["t"]
    ]
    will substitute a T = TFT_INT32 to TFT_PRODUCT[TFT_TENSOR[TFT_INT32]]
    and a T = (TFT_INT32, TFT_INT64) to
    TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_INT64]].
"""
TFT_CALLABLE: FullTypeId.ValueType  # 100
"""Callable types describe functions and ops.

Parametrization:
  TFT_CALLABLE[<arg type>, <return type>]
  * <arg type> is the type of the arguments; TFT_PRODUCT represents
  multiple
    arguments.
  * <return type> is the return type; TFT_PRODUCT represents multiple
    return values (that means that callables returning multiple things
    don't necessarily return a single tuple).

Example:
  TFT_CALLABLE[
    TFT_ANY,
    TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT64]],
  ]
    is a callable with unspecified (for now) input arguments, and
    two return values of type tensor.
"""
TFT_TENSOR: FullTypeId.ValueType  # 1000
"""Concrete type IDs, representing "proper" data types that can describe
runtime TensorFlow objects.

The usual Tensor. This is a parametric type.

Parametrization:
  TFT_TENSOR[<element type>, <shape type>]
  * <element type> is currently limited to one of the element types
    defined below.
  * <shape type> is not yet defined, and may only be TFT_UNKNOWN for now.

A TFT_SHAPE type will be defined in the future.

Example:
  TFT_TENSOR[TFT_INT32, TFT_UNKNOWN]
    is a Tensor of int32 element type and unknown shape.

TODO(mdan): Define TFT_SHAPE and add more examples.
"""
TFT_ARRAY: FullTypeId.ValueType  # 1001
"""Array (or tensorflow::TensorList in the variant type registry).
Note: this is not to be confused with the deprecated `TensorArray*` ops
which are not supported by FullType.
This type represents a random-access list whose elements can be
described by a single type. Although immutable, Array is expected to
support efficient mutation semantics (i.e. element update) in the
user-facing API.
The element type may be generic or even TFT_ANY for a heterogenous list.

Parametrization:
  TFT_ARRAY[<element type>]
  * <element type> may be any concrete type.

Examples:
  TFT_ARRAY[TFT_TENSOR[TFT_INT32]] is a TensorArray holding int32 Tensors
    of any shape.
  TFT_ARRAY[TFT_TENSOR[TFT_UNKNOWN]] is a TensorArray holding Tensors of
    mixed element types.
  TFT_ARRAY[TFT_UNKNOWN] is a TensorArray holding any element type.
  TFT_ARRAY[] is equivalent to TFT_ARRAY[TFT_UNKNOWN].
  TFT_ARRAY[TFT_ARRAY[]] is an array or arrays (of unknown types).
"""
TFT_OPTIONAL: FullTypeId.ValueType  # 1002
"""Optional (or tensorflow::OptionalVariant in the variant type registry).
This type represents a value that may either hold an element of a single
specified type, or nothing at all.

Parametrization:
  TFT_OPTIONAL[<element type>]
  * <element type> may be any concrete type.

Examples:
  TFT_OPTIONAL[TFT_TENSOR[TFT_INT32]] is an Optional holding an int32
    Tensor of any shape.
"""
TFT_LITERAL: FullTypeId.ValueType  # 1003
"""Literal types describe compile-time constant values.
Literal types may also participate in dependent types.

Parametrization:
  TFT_LITERAL[<value type>]{<value>}
  * <value type> may be any concrete type compatible that can hold <value>
  * <value> is the type's attribute, and holds the actual literal value

Examples:
  TFT_LITERAL[TFT_INT32]{1} is the compile-time constant 1.
"""
TFT_ENCODED: FullTypeId.ValueType  # 1004
"""Encoding types describe a value of a certain type, encoded as a different
type.

Parametrization:
  TFT_ENCODED[<encoded type>, <encoding type>]
  * <encoded type> may be any type
  * <encoding type> may be any type

Examples:
  TFT_ENCODING[TFT_INT32, TFT_STRING] is an integer encoded as string.
"""
TFT_SHAPE_TENSOR: FullTypeId.ValueType  # 1005
"""The type of "shape tensors" where the runtime value is the shape of
some tensor(s), i.e. the output of tf.shape.
Shape tensors have special, host-only placement, in contrast to
TFT_TENSOR[TFT_INT32] which is the type of a normal numeric tensor
with no special placement.

Examples:
  TFT_SHAPE_TENSOR[TFT_INT32] is the most common
  TFT_SHAPE_TENSOR[TFT_INT64] is also allowed
"""
TFT_BOOL: FullTypeId.ValueType  # 200
"""Type attributes. These always appear in the parametrization of a type,
never alone. For example, there is no such thing as a "bool" TensorFlow
object (for now).

The bool element type.
TODO(mdan): Quantized types, legacy representations (e.g. ref)
"""
TFT_UINT8: FullTypeId.ValueType  # 201
"""Integer element types."""
TFT_UINT16: FullTypeId.ValueType  # 202
TFT_UINT32: FullTypeId.ValueType  # 203
TFT_UINT64: FullTypeId.ValueType  # 204
TFT_INT8: FullTypeId.ValueType  # 205
TFT_INT16: FullTypeId.ValueType  # 206
TFT_INT32: FullTypeId.ValueType  # 207
TFT_INT64: FullTypeId.ValueType  # 208
TFT_HALF: FullTypeId.ValueType  # 209
"""Floating-point element types."""
TFT_FLOAT: FullTypeId.ValueType  # 210
TFT_DOUBLE: FullTypeId.ValueType  # 211
TFT_BFLOAT16: FullTypeId.ValueType  # 215
TFT_COMPLEX64: FullTypeId.ValueType  # 212
"""Complex element types.
TODO(mdan): Represent as TFT_COMPLEX[TFT_DOUBLE] instead?
"""
TFT_COMPLEX128: FullTypeId.ValueType  # 213
TFT_STRING: FullTypeId.ValueType  # 214
"""The string element type."""
TFT_DATASET: FullTypeId.ValueType  # 10102
"""Other types that we don't know yet whether they will become part of the
core type system or be consisdered third-party (and consequently moved to
user-defined type mechanisms). Presently, they are effectively in the core
type system, because key compilation passes like Placer account for their
existence.

Datasets created by tf.data ops and APIs. Datasets have generator/iterable
semantics, that is, one can construct an iterator from them. Like
Array, they are considered to return elements that can be described
by a single type. Unlike Array, they do not support random access or
mutation, and can potentially produce an infinite number of elements.
A datasets can produce logical structures (e.g. multiple elements). This
is expressed using TFT_PRODUCT.


Parametrization: TFT_DATASET[<element type>].
  * <element type> may be a concrete type or a type symbol. It represents
    the data type of the elements produced by the dataset.

Examples:
  TFT_DATSET[TFT_TENSOR[TFT_INT32]] is a Dataset producing single int32
    Tensors of unknown shape.
  TFT_DATSET[TFT_PRODUCT[TFT_TENSOR[TFT_INT32], TFT_TENSOR[TFT_FLOAT32]] is
    a Dataset producing pairs of Tensors, one integer and one float.
Note: The high ID number is to prepare for the eventuality that Datasets
will be supported by user types in the future.
"""
TFT_RAGGED: FullTypeId.ValueType  # 10103
"""A ragged tensor created by tf.ragged ops and APIs.

Parametrization: TFT_RAGGED[<element_type>].
"""
TFT_ITERATOR: FullTypeId.ValueType  # 10104
"""Iterators created by tf.data ops and APIs. Very similar to Datasets, except
they are mutable.


Parametrization: TFT_ITERATOR[<element type>].
  * <element type> may be a concrete type or a type symbol. It represents
    the data type of the elements produced by the dataset.
"""
TFT_MUTEX_LOCK: FullTypeId.ValueType  # 10202
"""A mutex lock tensor, produced by tf.raw_ops.MutexLock.
Unlike strict execution models, where ownership of a lock is denoted by
"running after the lock has been acquired", in non-strict mode, lock
ownership is in the true sense: "the op argument representing the lock is
available".
Mutex locks are the dynamic counterpart of control dependencies.
TODO(mdan): Properly document this thing.

Parametrization: TFT_MUTEX_LOCK[].
"""
TFT_LEGACY_VARIANT: FullTypeId.ValueType  # 10203
"""The equivalent of a Tensor with DT_VARIANT dtype, kept here to simplify
translation. This type should not normally appear after type inference.
Note that LEGACY_VARIANT != ANY: TENSOR[INT32] is a subtype of ANY, but is
not a subtype of LEGACY_VARIANT.
"""
global___FullTypeId = FullTypeId

@typing.final
class FullTypeDef(google.protobuf.message.Message):
    """Highly experimental and very likely to change.
    This encoding uses tags instead of dedicated messages for regularity. In
    particular the encoding imposes no restrictions on what the parameters of any
    type should be, which in particular needs to be true for type symbols.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_ID_FIELD_NUMBER: builtins.int
    ARGS_FIELD_NUMBER: builtins.int
    S_FIELD_NUMBER: builtins.int
    I_FIELD_NUMBER: builtins.int
    type_id: global___FullTypeId.ValueType
    """The principal type represented by this object. This may be a concrete type
    (Tensor, Dataset) a type variable (used for dependent types) a type
    symbol (Any, Union). See FullTypeId for details.
    """
    s: builtins.str
    i: builtins.int
    """TODO(mdan): list/tensor, map? Need to reconcile with TFT_RECORD, etc."""
    @property
    def args(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FullTypeDef]: ...
    def __init__(
        self,
        *,
        type_id: global___FullTypeId.ValueType | None = ...,
        args: collections.abc.Iterable[global___FullTypeDef] | None = ...,
        s: builtins.str | None = ...,
        i: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["attr", b"attr", "i", b"i", "s", b"s"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["args", b"args", "attr", b"attr", "i", b"i", "s", b"s", "type_id", b"type_id"]
    ) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["attr", b"attr"]) -> typing.Literal["s", "i"] | None: ...

global___FullTypeDef = FullTypeDef
